rm(list=ls())

########## !!!!!!!!     Apply this script, if each measured sample is in its own export file


###############################################
####
####    Script produce vectors from Genome Studio exports 
####    It is implied that the set of SNPs is ABSOLUTELY the same in all
####    export files and corresponds exactly to Annotation file created previously in script_0
####
####    As a result the folder Vectors_and_Segmentations will contain two vectors for each export file: 
####       V_LRR_Sample1, V_BAF_Sample1 
####
####    !!!! Name structure is essential!!!!
####    Sample name will be taken from SamplePlan file, if renaming exports
####
###############################################


############### File paths and other parameters


### !!!!! Put the PATH to your data  !!!!

pPath<-"C:/Users/Tatiana/Desktop/GAP_Illumina_scripts"
inputPath<-paste(pPath,"Exports",sep="/")
outputPath<-paste(pPath,"Vectors_and_segmentations",sep="/")


### !!!!! If you do not want to rename export files, comment the command renameExports<-TRUE   !!!!

renameExports<-FALSE
renameExports<-TRUE



### !!!!! If you want to rename export files, put the name of YOUR samplePlanFile  !!!!
### !!!!! In your sample plan, exports names should be in the second column, and the new names in the third one !!!!
### !!!!! In your sample plan, exports names should be in the second column, and the new names in the third one !!!!

samplePlanFile<-"Samples_plan_GAP_demo.csv"


### !!!!! Put your PATTERN="XXX", where XXX is some particular string from file names !!!!
### !!!!! Do not change it, if you process ALL files from Exports folder !!!!

PATTERN=""




####################################### START  ################################

############## Reading your exports file names

segFiles <- list.files(inputPath,pattern=PATTERN, full.names=T) 


fileNames <- gsub(inputPath,"",segFiles) 
fileNames <- gsub(".txt","",fileNames) 
#fileNames <- gsub("_DNA_tQN","",fileNames) 
#fileNames <- gsub("_ML_HS","",fileNames) 
fileNames <- gsub("/","",fileNames) 

fileNames

###### All Exports from "fileNames" will be processed !!!! Correct, if necessary

nSample<-length(fileNames)


############## Reading samples plan, if renaming export files

if(renameExports){samplePlan<-read.csv(paste(pPath,samplePlanFile,sep="/"),header=T,fill=T,stringsAsFactors = F)}


#############################################
############   MAIN CYCLE
#############################################


ii<-2

for(ii in 1:nSample){


if(renameExports){
	ii_name<-grep(fileNames[ii],samplePlan[,2])
	if(length(ii_name)!=1){
		if(length(ii_name)>1){
			ii_name1<-numeric(0)
			for(i in 1:length(ii_name)){
				if(length(grep(samplePlan[ii_name[i],2],fileNames[ii]))==1){ii_name1<-c(ii_name1,i)}
			}
			if(length(ii_name1)==1){fileNames[ii]<-as.character(samplePlan[ii_name[ii_name1],3])}
		}
	}else{if(length(grep(samplePlan[ii_name[1],2],fileNames[ii]))==1){fileNames[ii]<-as.character(samplePlan[ii_name[1],3])}}
}


print(ii)

### !!!!! Look at your export files !!!!!!
### !!!!! ID_column is the column with SNP IDs (need to be unique)
### !!!!!  correct the constant acording to your export file


ID_column<-1


#############  READING EXPORTS DEPENDING ON THE FORMAT 

#way 1, like in Affymetrix, each information row is marked by #
#aa<-scan(segFiles[ii],nlines=1,what="character",sep="\t")
#k<-1
#while((length(grep("#",aa))!=0)&&(grep("#",aa)==1)){
#aa<-scan(segFiles[ii],skip=k,nlines=1,what="character",sep="\t")
#k<-k+1
#}
#dataF<-read.delim(segFiles[ii],skip=k-1,header=T,fill=T,na.strings="null",row.names=ID_column)


## way 2, like in Illumina BeadSrudio exports, always 9 rows to skip

#k<-10
#dataF<-read.delim(segFiles[ii],skip=k-1,header=T,fill=T,na.strings=c("NA","NaN"),row.names=ID_column,as.is=T)


## way 3, GenomeStudio Exports

k<-1
dataF<-read.delim(segFiles[ii],skip=k-1,header=T,fill=T,na.strings=c("NA","NaN"),row.names=ID_column,as.is=T)




############################  !!!!!!!!!!! ######################################
#### !!!!!!      !!!!!!!!!
####
####     columns of dataF matrix should be adjusted to export file
####         column c_chr=1 : chromosome 
####         column c_pos=2 : position
####         column c_lrr=4 : Log2Ratio 
####         column c_baf=6  : B allele frequency
#### 
####  If you have other column order, please, correct them correspondingly 
####

colnames(dataF)

c_chr<-1
c_pos<-2
c_lrr<-3
c_baf<-4

#########################################################



Chr<-as.character(dataF[,c_chr])
rNames<-as.character(rownames(dataF))

print(table(Chr))

###### Chromosome renaming might need modification depending on what kind of chromosomes are on the chip:

tt<-which(Chr=="X");if(length(tt)>0){Chr[tt]<-23}
tt<-which(Chr=="Y");if(length(tt)>0){Chr[tt]<-24}
tt<-which(Chr=="XY");if(length(tt)>0){Chr[tt]<-25}
tt<-which(Chr=="MT");if(length(tt)>0){Chr[tt]<-26}
Chr<-as.numeric(Chr)

#######


dataF<-cbind(Chr,dataF[,c_pos],dataF[,c_lrr],dataF[,c_baf])
dataF<-data.matrix(dataF)
colnames(dataF)<-c("Chr_N","Position","LRR","BAF")

C_Chr<-1
C_Pos<-2
C_LRR<-3
C_BAF<-4

dataF[,C_LRR:C_BAF]<-round(dataF[,C_LRR:C_BAF],4)

dataF<-dataF[order(dataF[,C_Chr],dataF[,C_Pos],decreasing=F),]

dataFName<-paste(outputPath,paste("V_LRR_",as.character(fileNames[ii]),".txt",sep=""),sep="/")
write.table(dataF[,C_LRR], file = dataFName,quote=F,col.names=F,row.names=F)

dataFName<-paste(outputPath,paste("V_BAF_",as.character(fileNames[ii]),".txt",sep=""),sep="/")
write.table(dataF[,C_BAF], file = dataFName,quote=F,col.names=F,row.names=F)


}#ii

#######  END of MAIN CYCLE


#######################################   END  ###########################