rm(list=ls())

###############################################
####
#### Sript produce vectors from Genome Studio exports 
####    It is implied that the set of SNPs is ABSOLUTELY the same in all
####    export files and corresponds exactly to Annotation files created previously in script_0
####
####  As a result the folder Vectors_and_Segmentations will contain two vectors for each export file: 
####       V_LRR_Sample1, V_BAF_Sample1 
####
####   !!!! Name structure is essential!!!!
####    Sample name will be taken from SamplePlan file, if renaming exports
####
###############################################


############### File paths and other parameters


### !!!!! Put the PATH to your data  !!!!


pPath<-"C:/Users/Tatiana/Desktop/GAP_Illumina_scripts"

inputPath<-paste(pPath,"Exports",sep="/")
outputPath<-paste(pPath,"",sep="/")


############  Annotation file to be merged to the exports

SNP_annot_fileName<-"Illum660K_annot_cut.csv"


SNP_annot<-read.csv(paste(pPath,SNP_annot_fileName,sep="/"),header=T,fill=T,stringsAsFactors = F,row.names=1)
C_Ind<-1;C_Chr<-2;C_Pos<-3
nSNP<-dim(SNP_annot)[[1]]




#IDs<-read.delim(paste(inputPath,"Export_example.txt",sep="/"),header=T,fill=T,stringsAsFactors = F,sep="\t",row.names=1)

#tmp<-merge(IDs,SNP_annot,by.x=0,by.y=0)

#Ind_IDs<-setdiff(IDs[,1],tmp[,2])
#Ind_IDs<-IDs[Ind_IDs,]

#Ind_ann<-setdiff(SNP_annot[,1],tmp[,3])
3Ind_ann<-SNP_annot[Ind_ann,]



### !!!!! Put your PATTERN="XXX", where XXX is some particular string from file names !!!!
### !!!!! Do not change it, if you process ALL files from Exports folder !!!!
PATTERN="XXX_"




####################################### START  ################################

############## Reading your exports file names

segFiles <- list.files(inputPath,pattern=PATTERN, full.names=T) 

fileNames <- gsub(inputPath,"",segFiles) 
fileNames <- gsub(".txt","",fileNames) 
fileNames <- gsub("/","",fileNames) 

fileNames

nSample<-length(fileNames)

ii<-1


########## Pay attention to the columns of export file !!!!
col_LRR<-7
col_BAF<-8



for(ii in 1:nSample){
print(ii)


dataF<-read.delim(segFiles[ii],header=T,fill=T,stringsAsFactors = F,sep="\t",row.names=1)
dataF<-dataF[,c(col_LRR,col_BAF)]


tmp<-merge(dataF,SNP_annot,by.x=0,by.y=0)

rownames(tmp)<-tmp[,1]

tmp<-tmp[,-c(1,4)]
tmp<-cbind(tmp[,3:4],tmp[,1:2])

XX<-which(tmp[,1]==23)
YY<-which(tmp[,1]==24)
MM<-which(tmp[,1]==25)

tmp[XX,1]<-"X"
tmp[YY,1]<-"Y"
tmp[MM,1]<-"M"


dataFName<-paste(outputPath,"2_",as.character(fileNames[ii]),".txt",sep="")
write.table(tmp, file = dataFName,quote=F,col.names=T,row.names=T,sep="\t")


}#ii

fileNames <- colnames(dataF)[1]


##########################  !!!!!!    ############################

colnames(dataF)

c_lrr<-6
c_baf<-5
dataF[,c_lrr]<-round(dataF[,c_lrr],4)
dataF[,c_baf]<-round(dataF[,c_baf],4)


############################  !!!!!!!!!!! ######################################
#### !!!!!!      !!!!!!!!!
####
####     columns of dataF matrix should be adjusted to export file
####         column c_chr=1 : chromosome 
####         column c_pos=2 : position
####         column c_lrr=4 : Log2Ratio 
####         column c_baf=6  : B allele frequency
#### 
####  If you have other column order, please, correct them correspondingly 
####

#########################################################

if(dim(dataF)[[1]]==dim(SNP_annot)[[1]]){
dataF<-dataF[order(SNP_annot[,C_Ind],decreasing=F),]
}

#plot(dataF[1:10000,5])

dataFName<-paste(outputPath,paste("V_LRR_",as.character(fileNames),".txt",sep=""),sep="/")
write.table(dataF[,c_lrr], file = dataFName,quote=F,col.names=F,row.names=F)

dataFName<-paste(outputPath,paste("V_BAF_",as.character(fileNames),".txt",sep=""),sep="/")
write.table(dataF[,c_baf], file = dataFName,quote=F,col.names=F,row.names=F)


}#ii

#######  END of MAIN CYCLE


#######################################   END  ###########################