##################
#####
#####    This script provides the preparation workflow for copy number and genotype recognition
#####      in Illumina SNP arrays with the GAP method (Genome Biology, 2009, Popova et al)
#####      namesly, 
#####     1) segmentation of CN(copy number) and AD (allelic differences) profiles by CBS
#####     2) construction of segmented summary file for further analysis with GAP method
#####
#####    As an input we use vector forms of (1)Log R ratio and (2) B allele frequency, 
#####    As vector annotation we use IllumXXXk_SNP_short_annot.csv file   
#####
#####
#####    The input vectors from Vectors_and_Segmentations folder: V_LRR_Sample1,V_BAF_Sample1
#####   
#####    The output segmented summary files : segm_Sample1  are saved in Vectors_and_Segmentations folder
#####    
#####
##################

#rm(list=ls())

###############################################
### File paths and other parameters

### !!!!! Put the PATH to your data  !!!!

pPath<-"C:/Users/Tatiana/Desktop/GAP_Illumina_scripts"

inputPath<-paste(pPath,"Vectors_and_segmentations",sep="/")
outputPath<-paste(pPath,"Results_Plots",sep="/")


### Annotation file


SNP_annot_fileName<-"Illum660K_annot_cut.csv"


SNP_annot<-read.csv(paste(pPath,SNP_annot_fileName,sep="/"),header=T,fill=T,stringsAsFactors = F,row.names=1)
SNP_annot<-data.matrix(SNP_annot)
C_Ind<-1;C_Chr<-2;C_Pos<-3

nSNP<-dim(SNP_annot)[[1]]




samplePlanFile<-"Samples_plan_GAP_demo.csv"
samplePlan<-read.csv(paste(pPath,samplePlanFile,sep="/"),header=T,fill=T,stringsAsFactors = F)




### Source file need SNP_annot file defined above

source(paste(pPath,"script_functions_GAP_recognition_Illumina.R",sep="/"))


##################################################################
####
####  !!!! Cut-offs !!!! Correct, if necessary !!!!
####
####  germHomozyg.mBAF.thr is the most important cut-off for recognition
####  If using Affymetrix 100,250,500, etc, germHomozyg.mBAF.thr need to be around  0.85!!!!
####  This cut-off is used to eliminate homozygous SNPs
####     
####  germHet.Proportion is minimal proportion of heterozygous SNPs in the segment to be called Het
####  If the segment has less than germHet.Proportion heterozyg SNPs it is called Homozygous

germHomozyg.mBAF.thr<-0.95
germHet.Proportion<-(1/10)



#### !!!!  All V_LRR_Sample1 files form Vectors_and_Segmentations folder will be processed  !!!!


segFiles <- list.files(inputPath, pattern=vect_LR_pattern,full.names=T) 

fileNames <- gsub(inputPath,"",segFiles) 
fileNames <- gsub(vect_LR_pattern,"",fileNames) 
fileNames <- gsub(".txt","",fileNames) 
fileNames <- gsub("/","",fileNames) 

fileNames


nSample<-length(segFiles)


#########################    MAIN CYCLE   ########################

ii<-1

for(ii in 1:nSample){

print(ii)


tmp<-CBS_breakpoints_and_GAPconstruction(ii,signBAF=0.0001,signLRR=0.0001,germHomoThr=germHomozyg.mBAF.thr,germHetProp=germHet.Proportion)

#### signBAF and signLRR are p-values for segmentation algo CBS, less p-value, less breakpoints detected


plot(tmp[,c_baf],tmp[,c_lrr],cex=tmp[,c_len]/quantile(tmp[,c_len],prob=0.9),main=paste("GAP ",fileNames[ii]),
                xlab="Allelic Difference",ylab="Log ratio")
abline(v=0.5)
abline(h=0)


#### saving segm_File.txt with info line

	outputInfo<-paste("##","Annotations=",SNP_annot_fileName, ";germHomozyg_BAF_thr=",germHomozyg.mBAF.thr,";germHet_Proportion=",germHet.Proportion,sep="")

	fName<-paste(inputPath,paste(vect_Segm_pattern,fileNames[ii],".txt",sep=""),sep="/")
	write.table(outputInfo, file = fName,quote=F,row.names=F,col.names=F)
	write.table(tmp, file = fName,quote=F,row.names=F,sep="\t",append=T)

}#ii

#################### END MAIN CYCLE   ######################################







