######################################################################
######  Global parameters
######################################################################

##### Packages used

local({pkg <- "boot"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
local({pkg <- "MASS"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
local({pkg <- "mixtools"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
local({pkg <- "diptest"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
local({pkg <- "DNAcopy"; if(nchar(pkg)) library(pkg, character.only=TRUE)})
library(DNAcopy)


############### 1. FILE MASKs

# pattern of filename with Log ratio vector 
vect_LR_pattern="V_LRR_"

# pattern of filename with Allelic Differences vector 
vect_BAF_pattern="V_BAF_"

# pattern of filename for resulting segmented summary file 
vect_Segm_pattern="segm_"


############### 2. Constants

SR_perf<-c(-1,0,0.58,1,1.32,2)
pp_N<-seq(0,0.6,0.05)
#qq_L<-sort(c(seq(3,8,1)/32,seq(17,32,1)/64,seq(9,16,1)/16),decreasing=T)
qq_L<-sort(c(seq(3,8,1)/32,seq(17,32,1)/64),decreasing=T)

CN<-8

############## 3. Annotation of processing table

ColNamesTMP<-c("Index", "IndxSnpStart", "IndxSnpEnd",
 "Chr", "LenSnp", "LRR_BP", "BAF_BP", "LRR", "MajorBAF", "HomoSegm",
"nCN","nAD","nSB","Copy","Mallele","Confidence","Subclone",
"CN1","BA1","Rank1","CN2","BA2","Rank2","CN3","BA3","Rank3")

# "Index","IndxSnpStart","IndxSnpEnd","IndxCnStart","IndxCnEnd",
c_ind<-1; c_is<-2; c_if<-3; 

# "LenSnp","Chr",
c_chr<-4;c_len<-5; 

# "CN_BP" and "AD_BP"  
c_lrrBP<-6; c_bafBP<-7;

# "CN","AllDiff_L","AllDiff_H",
c_lrr<-8; c_baf<-9; c_bafH<-10


# "nCN","nBA","nSB"
c_nC<-11; c_nA<-12; c_nS<-13

# "Copy","Mallele","Confidence","Subclone",
c_CN<-14; c_BA<-15; c_conf<-16; c_sb<-17

# "CN1","BA1","Rank1","CN2","BA2","Rank2","CN3","BA3","Rank3" 
c_CN1<-18; c_BA1<-19; c_rnk1<-20




########################################################################
##################
################## Function getInfoSegm: reading and mining first line in proc_Sample file
##################
#########################################################################



getInfoSegm<-function(infoString){

tt<-unlist(strsplit(infoString,";"))

hh<-grep("p_BAF=",tt)
if(length(hh)==1){p_BAF<-as.numeric(sub("p_BAF=","",tt[hh]))}else{p_BAF<-NA}

hh<-grep("q_LRR=",tt)
if(length(hh)==1){q_LRR<-as.numeric(sub("q_LRR=","",tt[hh]))}else{q_LRR<-NA}

hh<-grep("2LRR=",tt)
if(length(hh)==1){Delta<-as.numeric(sub("2LRR=","",tt[hh]))}else{Delta<-NA}
Delta<-c(2,Delta)

hh<-grep("germHomozyg_BAF_thr=",tt)
if(length(hh)==1){germHomThr<-as.numeric(sub("germHomozyg_BAF_thr=","",tt[hh]))}else{germHomThr<-NA}


hh<-grep("germHet_Proportion=",tt)
if(length(hh)==1){germProp<-as.numeric(sub("germHet_Proportion=","",tt[hh]))}else{germProp<-NA}



out<-list(p_BAF=p_BAF,q_LRR=q_LRR,Delta=Delta,germHomThr=germHomThr,germProp=germProp)


}



readSegmFile<-function(segFileName){
	aa<-scan(segFileName,nlines=1,what="character",quiet = T)
	k<-1
	
	while((length(grep("#",aa))!=0)&&(grep("#",aa)==1)){
		aa<-scan(segFileName,skip=k,nlines=1,what="character",quiet = T)
		k<-k+1
	}
	
	tmp<-read.delim(segFileName,skip=k-1, header=T,sep="\t")
	tmp<-data.matrix(tmp)
	if(k==2){
		results<-getInfoSegm(scan(segFileName,nlines=1,what="character",quiet = T))
		
		p_BAF<-results$p_BAF
		q_LRR<-results$q_LRR
		Delta<-results$Delta
		germHomThr<-results$germHomThr
		germProp<-results$germProp

	}else{p_BAF<-NA;q_LRR<-NA;Delta<-c(2,NA);germHomThr<-NA;germProp<-NA}

out<-list(tmp=tmp,p_BAF=p_BAF,q_LRR=q_LRR,Delta=Delta,germHomThr=germHomThr,germProp=germProp)
}



################################
#######
#####     Function calculates a table of Snps in chromosomes
#######
###################################

getRecCHR<-function(){
tt<-table(SNP_annot[,C_Chr],SNP_annot[,C_Chr+1])
CHR<-c(0);tCHR<-c(0)

for(k in 1:dim(tt)[[1]]){
	CHR<-c(CHR,CHR[2*k-1]+tt[k,1]);CHR<-c(CHR,CHR[2*k]+tt[k,2])
	tCHR<-c(tCHR,as.numeric(rownames(tt))[k])
	tCHR<-c(tCHR,as.numeric(rownames(tt))[k]+0.5)
}

names(CHR)<-tCHR;rm(tCHR);
out<-CHR

}


#######################################################################
##################
################## Function getBAFforRecognition
##################
#########################################################################

getBAFforRecognition<-function(tmp,datA,germHomoThr,germHetProp){

tt<-which(is.na(datA));if(length(tt)!=0){datA[tt]<-1}
tt<-which((datA>1));if(length(tt)!=0){datA[tt]<-1}

ad<-datA
tt<-which(datA>germHomoThr);if(length(tt)!=0){ad[tt]<-NA}

k<-14
for(k in 1:dim(tmp)[[1]]){

  adv<-ad[tmp[k,c_is]:tmp[k,c_if]]
  tt<-which(!is.na(adv))

  if(length(tt)>length(adv)*germHetProp){

	adv<-adv[tt]
	if(length(adv)>30){

	  tmp[k,c_baf]<-median(adv,na.rm=T)
	  if(tmp[k,c_baf]<0.6){
		hh<-hist(adv, breaks=seq(0.5,1,0.02),right=F,plot=F)
		
		if((sum(hh$counts[1:3])/sum(hh$counts)>0.7)&&((hh$counts[1]+hh$counts[2])>2*hh$counts[3]*1.2)){
			if(hh$counts[1]>hh$counts[2]){tmp[k,c_baf]<-0.5}else{tmp[k,c_baf]<-0.52}
		}else{dd<-density(adv,n=20)
			tmp[k,c_baf]<-dd$x[which.max(dd$y)]
			}
	  }
	
	}else{if(length(adv)>0){hh<-hist(adv, breaks=seq(0.5,1,0.05),plot=F)
			tmp[k,c_baf]<-hh$mids[which.max(hh$counts)]
		}else{tmp[k,c_baf]<-NA}
		}

  }else{tt<-which(is.na(adv))
	adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
	if(length(tt)>10){
		adv<-adv[tt]
		if(length(adv)>30){
			dd<-density(adv,n=20)
			tmp[k,c_baf]<-dd$x[which.max(dd$y)]
		}else{hh<-hist(adv, breaks=seq(0.5,1,0.05),right=F,plot=F)
			tmp[k,c_baf]<-hh$mids[which.max(hh$counts)]}
	}else{
		if(length(tt)>0){
			adv<-adv[tt]
			hh<-hist(adv, breaks=seq(0.5,1,0.1),plot=F)
			tmp[k,c_baf]<-hh$mids[which.max(hh$counts)]
		}else{tmp[k,c_baf]<-NA}
		}
	}
 
}#k


tmp<-round(tmp,4)
out<-tmp

}#function


###############################################################
#####
#####  Main function for script_3...
#####
################################################################

CBS_breakpoints_and_GAPconstruction<-function(ii,signBAF,signLRR,germHomoThr,germHetProp){

if((signBAF>0.05)|(signBAF<10^(-10))){signBAF<-0.0001}
if((signLRR>0.05)|(signLRR<10^(-10))){signLRR<-0.0001}


######## segmented table annotation

ColNamesTMP<-c("Index", "IndxSnpStart", "IndxSnpEnd",
 "Chr", "LenSnp", "LRR_BP", "BAF_BP", "LRR", "MajorBAF", "homoSegm")

c_ind<-1;c_is<-2;c_if<-3;c_chr<-4;c_len<-5;c_lrrBP<-6;c_bafBP<-7;c_lrr<-8;c_baf<-9;c_homo<-10



dataF<-matrix(0,nSNP,4)
colnames(dataF)<-c("BP_CN","BP_AD","AD_hete","Homo_segm")
c_bpLRR<-1;c_bpBAF<-2;c_hetAD<-3;c_homo<-4

##################################################################
####
####  Applying CBS algorithm to LRR
####
##################################################################

datL<-read.table(segFiles[ii],header=F)
datL<-data.matrix(datL)
datL<-as.numeric(datL)
tt<-which(abs(datL)>100);if(length(tt)>0){datL[tt]<-NA}

tt<-which(datL<(-1));if(length(tt)>0){datL[tt]<-(-datL[tt])/(datL[tt]-1)-0.5}


notNAs<-which(!is.na(datL))
CNA.object <- CNA(datL[notNAs],round((SNP_annot[notNAs,C_Chr]+(SNP_annot[notNAs,C_Chr+1]-1)/2),1),SNP_annot[notNAs,C_Ind],data.type="logratio",sampleid=ii)

CNA.object <- smooth.CNA(CNA.object, smooth.region=2, outlier.SD.scale=2.5, smooth.SD.scale=2,trim=0.1)

segment.CNA.object <- segment(CNA.object,alpha = signLRR, nperm = 1000, p.method = "hybrid", min.width=5, kmax=25, nmin=200, 
       eta=0.05, trim = 0.025, undo.splits = "none", verbose=1)

tmp <- segment.CNA.object$output

rm(segment.CNA.object,CNA.object)

dataF[tmp$loc.end,c_bpLRR]<-1

rm(tmp)

##################################################################
####
####  Applying CBS algorithm to BAF
####
##################################################################


datA<-read.table(sub(vect_LR_pattern,vect_BAF_pattern,segFiles[ii]),header=F)
datA<-data.matrix(datA)
datA<-as.numeric(datA)

tt<-which(abs(datA)>1);if(length(tt)>0){datA[tt]<-1}
tt<-which(is.na(datA));if(length(tt)>0){datA[tt]<-1}

datA<-abs(datA-0.5)+0.5

########################################
####
#### 1. Marking germline homozygous SNPs and regions
####
########################################

#### scaling size
if(length(datA)/100<200){gHL<-40;gHS<-8}
if(length(datA)/100<400){gHL<-50;gHS<-10}
if(length(datA)/100<700){gHL<-80;gHS<-16}
if(length(datA)/100>=700){gHL<-100;gHS<-20}

homoA<-datA*0
tt<-which(abs(datA)>germHomoThr)
if(length(tt)>0){homoA[tt]<-1}

S_homo<-homoA
rh<-S_homo*0
lh<-S_homo*0
for(k in 1:(gHS-1)){
lh[gHS:length(lh)]<-lh[gHS:length(lh)]+S_homo[(gHS-1-k+1):(length(S_homo)-k)]
rh[1:(length(rh)-gHS+1)]<-rh[1:(length(rh)-gHS+1)]+S_homo[(k+1):(length(S_homo)-gHS+1+k)]
}

S_hete<-S_homo*0
tt<-which(rh+lh<gHS*1.5);if(length(tt)>0){S_hete[tt]<-1}
tt<-which(rh<(gHS*0.9));if(length(tt)>0){S_hete[tt]<-1}
tt<-which(lh<(gHS*0.9));if(length(tt)>0){S_hete[tt]<-1}

S_homo<-(1-S_hete)

rh<-S_homo*0
lh<-S_homo*0
for(k in 1:(gHL-1)){
lh[gHL:length(lh)]<-lh[gHL:length(lh)]+S_homo[(gHL-1-k+1):(length(S_homo)-k)]
rh[1:(length(rh)-gHL+1)]<-rh[1:(length(rh)-gHL+1)]+S_homo[(k+1):(length(S_homo)-gHL+1+k)]
}
S_homo<-S_homo*0
tt<-which(rh+lh>gHL*1.8);if(length(tt)>0){S_homo[tt]<-1}
tt<-which(rh>gHL*0.9);if(length(tt)>0){S_homo[tt]<-1}
tt<-which(lh>gHL*0.9);if(length(tt)>0){S_homo[tt]<-1}

S_homo<-S_homo*(1-S_hete)

seg<-abs(S_homo[2:nSNP]-S_homo[1:(nSNP-1)])

seg<-c(1,which(seg!=0),nSNP)
seg<-as.numeric(rownames(table(seg)))

for(k in 1:(length(seg)-1)){
if(S_homo[seg[k]+1]==0){
if(sum(1-homoA[(seg[k]+1):seg[k+1]])<5){S_homo[(seg[k]+1):seg[k+1]]<-1}
}}

seg<-abs(S_homo[2:nSNP]-S_homo[1:(nSNP-1)])+
	abs(SNP_annot[2:nSNP,C_Chr]-SNP_annot[1:(nSNP-1),C_Chr])+
	abs(SNP_annot[2:nSNP,C_Chr+1]-SNP_annot[1:(nSNP-1),C_Chr+1])

seg<-c(1,which(seg!=0),nSNP)
seg<-as.numeric(rownames(table(seg)))

for(k in 2:length(seg)){
if(seg[k]-seg[k-1]+1<gHL*2){S_homo[seg[k-1]:seg[k]]<-0}
}

dataF[,c_homo]<-S_homo
dataF[,c_hetAD]<-(1-homoA)

########################################
####
#### 2. Segmenting BAF by CBS
####
########################################

tmp<-cbind(datA,round((SNP_annot[,C_Chr]+(SNP_annot[,C_Chr+1]-1)/2),1),SNP_annot[,C_Ind])
tmp<-data.matrix(tmp)

tt<-which((1-dataF[,c_hetAD])+dataF[,c_homo]==0);
if(length(tt)>0){tmp<-tmp[tt,]}

CNA.object <- CNA(tmp[,1],tmp[,2],tmp[,3],data.type="logratio",sampleid=ii)
CNA.object <- smooth.CNA(CNA.object, smooth.region=4, outlier.SD.scale=4, smooth.SD.scale=1,trim=0.05)

segment.CNA.object <- segment(CNA.object,alpha = signBAF, nperm = 1000, p.method = "hybrid", min.width=5, kmax=25, nmin=200, 
       eta=0.05, trim = 0.025, undo.splits = "none", verbose=1)

tmp <- segment.CNA.object$output
rm(segment.CNA.object,CNA.object)

dataF[tmp$loc.end,c_bpBAF]<-1
rm(tmp)


##################################################
################## Constructing GAP

seg<-abs(SNP_annot[1:(nSNP-1),C_Chr]-SNP_annot[2:(nSNP-0),C_Chr])+
	abs(SNP_annot[1:(nSNP-1),C_Chr+1]-SNP_annot[2:(nSNP-0),C_Chr+1])+
	dataF[1:(nSNP-1),c_bpLRR]+dataF[1:(nSNP-1),c_bpBAF]

seg<-which(seg!=0)
seg<-c(seg,nSNP)
seg<-cbind(data.matrix(SNP_annot[seg,]),dataF[seg,])

tmp<-matrix(0,dim(seg)[[1]],10)
colnames(tmp)<-ColNamesTMP

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])

tmp[,c_lrrBP]<-seg[,c_bpLRR+4]
tmp[,c_bafBP]<-seg[,c_bpBAF+4]

tmp[,c_chr]<-seg[,C_Chr]+(seg[,C_Chr+1]-1)/2
tmp[,c_if]<-seg[,C_Ind]
tmp[,c_is]<-c(1,seg[1:(dim(seg)[[1]]-1),C_Ind]+1)
tmp[,c_len]<-tmp[,c_if]-tmp[,c_is]+1


#################################################
####
############### Adjusting between chromosome and arm connections
####
#################################################

Flag<-T
while(Flag){
Flag<-F
for(k in 2:(dim(tmp)[[1]]-2)){
	if(tmp[k,c_chr]!=tmp[k+1,c_chr]){
		if((tmp[k,c_len]<5)&&(tmp[k,c_chr]==tmp[k-1,c_chr])){
			tmp[k-1,c_if]<-tmp[k,c_if]
			tmp[k-1,c_len]<-tmp[k-1,c_len]+tmp[k,c_len]
			tmp[k,c_len]<-0
		}
		if((tmp[k+1,c_len]<5)&&(tmp[k+1,c_chr]==tmp[k+2,c_chr])){
			tmp[k+2,c_is]<-tmp[k+1,c_is]
			tmp[k+2,c_len]<-tmp[k+1,c_len]+tmp[k+2,c_len]
			tmp[k+1,c_len]<-0
		}
	}
}#k

if((tmp[1,c_len]<5)&&(tmp[1,c_chr]==tmp[2,c_chr])){
	tmp[2,c_is]<-tmp[1,c_is]
	tmp[2,c_len]<-tmp[1,c_len]+tmp[2,c_len]
	tmp[1,c_len]<-0
}

tt<-dim(tmp)[[1]]
if((tmp[tt,c_len]<5)&&(tmp[tt,c_chr]==tmp[tt-1,c_chr])){
	tmp[tt-1,c_if]<-tmp[tt,c_if]
	tmp[tt-1,c_len]<-tmp[tt-1,c_len]+tmp[tt,c_len]
	tmp[tt,c_len]<-0
}

tt<-which(tmp[,c_len]==0)
if(length(tt)!=0){tmp<-tmp[-tt,];Flag<-T}

}#Flag

#################################################"
############ Calculating LRR and BAF modes

k<-1
for(k in 1:dim(tmp)[[1]]){
	if(tmp[k,c_if]>tmp[k,c_is]){
	tmp[k,c_lrr]<-median(datL[tmp[k,c_is]:tmp[k,c_if]],na.rm=T)
	}else{tmp[k,c_lrr]<-datL[tmp[k,c_is]]}	
}#k


#c_bpLRR<-1;c_bpBAF<-2;c_hetAD<-3;c_homo<-4

tmp<-getBAFforRecognition(tmp,datA,germHomoThr,germHetProp)

tmp<-round(tmp,4)

out<-tmp

}#function





#######################################################################
##################
################## Function smoothNET
##################
#########################################################################



smoothNET<-function(NET,weightC){

NR<-dim(NET)[[1]]
NF<-dim(NET)[[2]]

k<-1
for(k in 1:2){
	NET1<-NET
	for(j in 2:(NF-1)){
		for(p in 2:(NR-1)){
			NET1[p,j]<-(sum(NET[(p-1):(p+1),(j-1):(j+1)])+weightC*NET[p,j])/(9+weightC)
			if((NET[p,j]>50)&&(NET1[p,j]<50)){NET1[p,j]<-NET[p,j]}
		}#p
		NET1[1,j]<-sum(NET[1:2,(j-1):(j+1)])/6
		NET1[NR,j]<-sum(NET[(NR-1):NR,(j-1):(j+1)])/6
	}#j
	for(p in 2:(NR-1)){
		NET1[p,1]<-sum(NET[(p-1):(p+1),1:2])/6
		NET1[p,NF]<-sum(NET[(p-1):(p+1),(NF-1):NF])/6
	}#p

	NET1[1,1]<-(NET[1,1]+NET[2,1]+NET[1,2]+NET[2,2])/4
	NET1[NR,NF]<-(NET[NR,NF]+NET[NR-1,NF]+NET[NR,NF-1]+NET[NR-1,NF-1])/4

	NET<-NET1

k}#k

rm(NET1)
out<-NET
out
}



#########################################################################
##################
################## Function Centr_CL_annot
##################
#########################################################################


Centr_CL_annot <- function(CN){

CC<-matrix(0,2,2)
k<-2
for(cn in 2:CN){CC<-cbind(CC,matrix(0,2,k))
	if(cn%%2==1){k<-k+1}
}

for(cn in 1:(CN+1)){
	if(cn-1<=2){cn_CL<-cn-1}

	if(cn-1>2){
		cn_CL<-2
		for(k in 2:(cn-1)){
			for(nbc in trunc(k/2+1):k){cn_CL<-cn_CL+1}
		}
		cn_CL<-cn_CL-1
	}

	for(nbc in 1:trunc((cn-1)/2+1)){
	CC[1,cn_CL+nbc]<-cn-1
	CC[2,cn_CL+nbc]<-trunc((cn-1+1)/2)+nbc-1
	}
}# cn

CC<-rbind(CC,seq(1,dim(CC)[[2]]))
CC<-rbind(CC,matrix(0,2,dim(CC)[[2]]))

rownames(CC)<-c("Copy","Major allele","Index","BAF","LRR")
out<-CC
}


#########################################################################
##################
################## Function plotGAPdensity
##################
#########################################################################


plotGAPdensity<-function(NET){

NR<-dim(NET)[[1]]
NF<-dim(NET)[[2]]
SR<-as.numeric(rownames(NET))
SF<-as.numeric(colnames(NET))

plot(c(SF[1],SF[NF]),c(max(-1,SR[1]),SR[NR]),type="n",main="GAP",xlab="major B allele Frequency",ylab="LogR ratio",xlim=c(0.5,1))


NETM<-as.numeric(NET)
NETM<-cbind(NETM,NETM,NETM)
NETM[,1]<-rep(seq(1,length(SR)),length(SF))
NETM[,2]<-rep(seq(1,length(SF)),each=length(SR))
NETM<-NETM[-which(NETM[,3]<10),]
NETM[,3]<-log2(NETM[,3])
NETM<-cbind(NETM,NETM[,3])
NETM<-NETM[order(NETM[,3],decreasing=F),]


hh<-hist(NETM[,3],breaks=64,plot=F)$breaks

p<-1
k<-1
while(p<=length(hh)){
while((k<dim(NETM)[[1]])&&(NETM[k,3]<hh[p])){
NETM[k,4]<-p
k<-k+1
}
p<-p+1
}

tt<-which(NETM[1,]==NR)
if(length(tt)>0){NETM[1,tt]<-NR-1}

tt<-which(NETM[2,]==NF)
if(length(tt)>0){NETM[2,tt]<-NF-1}

RB<-rainbow(length(hh))
RB[1]<-"#FF000000"

for(p in 1:dim(NETM)[[1]]){
rect(SF[NETM[p,2]],SR[NETM[p,1]],SF[NETM[p,2]+1],SR[NETM[p,1]+1], col=RB[NETM[p,4]],border=RB[NETM[p,4]])
}

}




#########################################################################
##################
################## Function fittingGAP
##################
#########################################################################


fittingGAP <- function (NET){

NF<-dim(NET)[[2]]
NR<-dim(NET)[[1]]
SF<-as.numeric(colnames(NET))
SR<-as.numeric(rownames(NET))
step_SF<-SF[2]-SF[1]
step_SR<-SR[2]-SR[1]

step_L<-0.005
Start<-which.min(abs(SR+1))

lim_CL<-3

#### Initializing ouput matrix FF_all, with cluster position and quality criterium

FF_all<-matrix(0,24,2)

rownames(FF_all)<-c("Coverage","p_BAF","p_LRR","1","2","3","4","5",
"copy_2","clust34","homo_clust","5_3","4_0","4_4","4_3","4_2","3_0","3_3","3_2","2_0","2_2","2_1","1_0","1_1")

#### Cycle for fitting tumor DNA proportion


jj<-1
for(jj in 1:length(pp_N)){

	NET_C<-matrix(0,6,NF)

	rownames(NET_C)<-SR_perf
	colnames(NET_C)<-seq(1,NF,1)

	tt<-0
	for(cn in 1:5){
		nbn<-1
		if(cn%%2==0){NET_C[cn,1]<-1}
		
		for(nbc in trunc(cn/2+1):cn){
		baf<-((1-pp_N[jj])*nbc+pp_N[jj]*nbn)/((1-pp_N[jj])*cn+2*pp_N[jj])

		if((cn==2)&&(nbc==cn)&&(baf<=0.98)){tt<-1}

		if((cn>1)&&(nbc==cn)&&(tt!=0)){
			NET_C[cn,min(95,which.min(abs(SF-baf)))]<-1
			}else{NET_C[cn,which.min(abs(SF-baf))]<-1}
		}#nbc
	
		nbn<-2
		nbc<-cn
		baf<-((1-pp_N[jj])*nbc+pp_N[jj]*nbn)/((1-pp_N[jj])*cn+2*pp_N[jj])
		NET_C[cn,(dim(NET_C)[[2]]-3)]<-1

	}#cn

	NET_C[6,]<-NET_C[1,]+NET_C[2,]+NET_C[3,]+NET_C[4,]+NET_C[5,]
	NET_C<-NET_C[,-which(NET_C[6,]==0)]
	NET_C[6,]<-as.numeric(colnames(NET_C))

	NET_tmp<-matrix(1,16,7)
	colnames(NET_tmp)<-c("N_LRR","N_BAF","Step_LRR","Step_BAF_le","Step_BAF_ri","Copy","majorA")
	NET_tmp[,6]<-c(5,5,5,5,4,4,4,4,3,3,3,2,2,2,1,1)
	NET_tmp[,7]<-c(0,5,4,3,0,4,3,2,0,3,2,0,2,1,0,1)

#### Cycle for fitting koefficient of LRR shrinkage
	kk<-1
	for(kk in 1:length(qq_L)){

		SR_C<-SR_perf*qq_L[kk]

		for(k in 1:length(SR_C)){
			SR_C[k]<-round((SR_C[k]-SR[1])/step_L+1,0)
		}#k
		
		SR_C<-SR_C-SR_C[2]+Start

#### Determination GAP structure, such as cluster size in BAf 
#### and LRR scale for given coefficients

		tt<-as.numeric(NET_C[1:5,])
		tt<-cbind(matrix(0,length(tt),6),tt)
		tt[,1]<-rep(SR_C[1:5],dim(NET_C)[[2]])
		tt[,3]<-rep(c(max(SR_C[2:5]-SR_C[1:4]),SR_C[2:5]-SR_C[1:4]),dim(NET_C)[[2]])
		tt[,2]<-rep(NET_C[6,],each=5)

		tt<-tt[-which(tt[,7]==0),]
		tt<-tt[order(tt[,1],tt[,2],decreasing=T),]

		NET_tmp[,1:5]<-tt[,1:5]


		NET_tmp[,3]<-round(NET_tmp[,3]/lim_CL,0)
		tt<-which(NET_tmp[,3]>20);if(length(tt)>0){NET_tmp[tt,3]<-20}
		tt<-which(NET_tmp[,3]<6);if(length(tt)>0){NET_tmp[tt,3]<-round(NET_tmp[tt,3]*3/2-0.6)}

		NET_tmp[1:(dim(NET_tmp)[[1]]-1),4]<-NET_tmp[1:(dim(NET_tmp)[[1]]-1),2]-NET_tmp[2:dim(NET_tmp)[[1]],2]-1
		tt<-which(NET_tmp[,4]<0); if(length(tt)>0) {NET_tmp[tt,4]<-NET_tmp[tt,2]-1}
		NET_tmp[dim(NET_tmp)[[1]],4]<-max(0,NET_tmp[dim(NET_tmp)[[1]],2]-1)

		NET_tmp[2:dim(NET_tmp)[[1]],5]<-NET_tmp[1:(dim(NET_tmp)[[1]]-1),2]-NET_tmp[2:dim(NET_tmp)[[1]],2]-1
		tt<-which(NET_tmp[,5]<0); if(length(tt)>0) {NET_tmp[tt,5]<-2*(NF-NET_tmp[tt,2])}
		NET_tmp[1,5]<-2*max(1,NF-NET_tmp[1,2])

		NET_tmp[,4]<-NET_tmp[,4]/2
		NET_tmp[,5]<-NET_tmp[,5]/2

		NET_tmp[1,4]<-min(NET_tmp[1,4],5)
		if(NET_tmp[2,2]>95){NET_tmp[2,4]<-min(NET_tmp[2,4],10)}
		NET_tmp[2,5]<-min(max(min(NET_tmp[2,5],5),NET_tmp[2,5]/2),10)*2

		for(k in 2:dim(NET_tmp)[[1]]){
			if(NET_tmp[k,2]==102){NET_tmp[k,4]<-min(NET_tmp[k,4],5)
				NET_tmp[k+1,5]<-min(max(min(NET_tmp[k+1,5],5),NET_tmp[k+1,5]/2),10)*2
				NET_tmp[k-1,4]<-min(NET_tmp[k-1,4]*2,10)
				if(NET_tmp[k+1,2]>95){NET_tmp[k+1,4]<-min(NET_tmp[k+1,4],10)}
			}else{
				if(NET_tmp[k,2]==1){NET_tmp[k,5]<-min(NET_tmp[k,5]*2/3,15)
				}else{NET_tmp[k,5]<-min(NET_tmp[k,5]/2,10)}
				NET_tmp[k,4]<-min(NET_tmp[k,4]/2,10)
				}
		}#k

		NET_tmp<-round(NET_tmp+0.1,0)

		NET_tmp[,4]<-NET_tmp[,2]-NET_tmp[,4]
		NET_tmp[,5]<-NET_tmp[,2]+NET_tmp[,5]

		tt<-which(NET_tmp[,4]<1);if(length(tt)>0){NET_tmp[tt,4]<-1}
		tt<-which(NET_tmp[,5]>NF);if(length(tt)>0){NET_tmp[tt,5]<-NF}

		pos<-length(which(NET_tmp[,1]>0))
		outr<-length(which(NET_tmp[,1]>NR))

		FF_max2<-rep(0,24)
		FF_max4<-rep(0,24)

#### Cycle for fitting GAP pattern position on NET

		while(SR_C[2]<NR){

			FF<-rep(0,24)

			for(j in (outr+1):pos){
				FF[8+j]<-sum(NET[max(1,(NET_tmp[j,1]-NET_tmp[j,3])):min(NR,(NET_tmp[j,1]+NET_tmp[j,3])),NET_tmp[j,4]:NET_tmp[j,5]])
			}#j
			FF[1]<-sum(FF)
			if(FF[1]>FF_max4[1]){
				FF_max4<-c(FF[1],pp_N[jj],qq_L[kk],SR_C[1:5],FF[9:24])
			}

			FF[1]<-sum(FF[17:24])
			if((-FF[1])<FF_max2[1]){
				FF_max2<-c(-FF[1],pp_N[jj],qq_L[kk],SR_C[1:5],FF[9:24])
			}



			NET_tmp[,1]<-NET_tmp[,1]+1
			SR_C<-SR_C+1

			if((pos<dim(NET_tmp)[[1]])&&(NET_tmp[pos+1,1]>0)){
			pos<-length(which(NET_tmp[,1]>0))}

			if((outr<dim(NET_tmp)[[1]])&&(NET_tmp[outr+1,1]>NR)){
			outr<-length(which(NET_tmp[,1]>NR))}


		}#SR_C

		if(FF_max2[1]<0){FF_all<-cbind(FF_all,FF_max2)}
		if(FF_max4[1]>0){FF_all<-cbind(FF_all,FF_max4)}

	}#kk
	
}#jj

#######################################################
#### Filtering resulting patterns

#### 1. Set clusters that are beyond the NET to 0
for(j in 4:8){
	if(length(which(FF_all[j,]<0))>0){FF_all[j,which(FF_all[j,]<0)]<-0}
	if(length(which(FF_all[j,]>NR))>0){FF_all[j,which(FF_all[j,]>NR)]<-0}
}

FF_all[9,]<-FF_all[20,]+FF_all[21,]+FF_all[22,]
FF_all[10,]<-FF_all[15,]+FF_all[19,]
FF_all[11,]<-FF_all[14,]+FF_all[18,]+FF_all[21,]+FF_all[24,]
FF_all[12,]<-sign(FF_all[14,]*FF_all[15,]*FF_all[18,]*FF_all[19,]*FF_all[2,])*500

#### 2. Removing patterns that either has no 2 copy level or no 3/2 and 4/3

#tt<-which(FF_all[9,]+FF_all[10,]<500); if(length(tt)>0){FF_all<-FF_all[,-tt]}

tt<-which(FF_all[9,]+FF_all[12,]<500); if(length(tt)>0){FF_all<-FF_all[,-tt]}

#### 3. Removing patterns that has nothing at tumor homozygous line, if no 3/2 cluster

tt<-which(FF_all[10,]>500);if(length(tt)>0){FF_all[10,tt]<-1000}

tt<-which(FF_all[11,]-FF_all[10,]<0); 
ss<-which(FF_all[19,]>2000); 
tt<-setdiff(tt,ss)
if(length(tt)>0){FF_all<-FF_all[,-tt]}

#### 4. Order patterns by COVERAGE criterium and leave the first 200 solution
#### for further analysis

FF_all<-FF_all[1:11,]

tt<-which(FF_all[1,]<0)
if(length(tt)>0){FF_all2<-FF_all[,tt]
if(dim(FF_all)[[2]]>length(tt)){FF_all<-FF_all[,-tt]}else{
FF_all<-FF_all2
}

}else{FF_all2<-NA}


FF_all<-abs(FF_all)

#FF_all<-abs(FF_all2)

FF_all<-FF_all[,order(FF_all[1,],decreasing=T)]


######################## Summarizing FF_all

FF_all[9,]<-0
FF_all[10,]<-round(FF_all[1,]/FF_all[1,1]*100,1)
tt<-which(FF_all[10,]<70)
if(length(tt)>0){FF_all<-FF_all[,-tt]}


for(k in 1:(dim(FF_all)[[2]]-1)){
	if(FF_all[9,k]==0){FF_all[9,k]<-k;kn<-1
		
		for(kk in (k+1):dim(FF_all)[[2]]){
			if((max(abs(FF_all[5:7,k]-FF_all[5:7,kk]))<10)&&
				(abs(FF_all[2,k]-FF_all[2,kk])<0.1)){
				if((FF_all[10,k]-FF_all[10,kk]<10)&&(kn<10)){FF_all[9,kk]<-k
                        kn<-kn+1
                        }else{FF_all[9,kk]<-(-1)}
			}else{}
	}}
}

k<-dim(FF_all)[[2]]
if(FF_all[9,k]==0){FF_all[9,k]<-k}


tt<-which(FF_all[9,]==(-1))
if(length(tt)>1){
if(length(tt)>0){FF_all<-FF_all[,-tt]}
}else{FF_all<-cbind(FF_all,FF_all)
tt<-which(FF_all[9,]==(-1))
if(length(tt)>0){FF_all<-FF_all[,-tt]}
}


FF_all<-FF_all[,order(FF_all[9,],decreasing=F)]

#table(FF_all[9,])

FF_all<-FF_all[1:10,]
rownames(FF_all)<-c("Coverage_SNP","p_BAF","q_LRR","1_copy","2_copy","3_copy","4_copy","5_copy","Set_pattern","Percent_coverage")

out<-FF_all


}# function fit GAP


#########################################################################
##################
################## Function getCenterClusterGAP
##################
#########################################################################

getCenterClusterGAP<-function(CN, pp, qq, Delta){

	Centr_CL<-Centr_CL_annot(CN)
	
	SR_perf<-(-4)
	for(cn in 1:8){SR_perf<-c(SR_perf,log2(1.1*cn/2))}

	SR_perf<-SR_perf*qq
	
	if(SR_perf[6]-SR_perf[5]<0.1){
		for(cn in 7:9){
		SR_perf[cn:9]<-SR_perf[cn:9]-(SR_perf[cn]-SR_perf[cn-1])/2+(SR_perf[6]-SR_perf[5])/2
		}
	}

	DD<-SR_perf[Delta[1]+1]-Delta[2]
	
	SR_perf<-round((SR_perf-DD),4)

	for(cn in 1:9){Centr_CL[5,which(Centr_CL[1,]==(cn-1))]<-SR_perf[cn]}

	Centr_CL[4,1]<-0.5
	nbn<-1

	for(cn in 1:8){
		max_F<-rep(0,cn+1)
			if(cn%%2==0){max_F[cn/2+1]<-0.5}

			for(nbc in trunc(cn/2+1):cn){
				baf<-((1-pp)*nbc+pp*nbn)/((1-pp)*cn+2*pp)
				max_F[nbc+1]<-round(baf,4)
			}#nbc

			cn_CL<-1
			while(Centr_CL[1,cn_CL]<cn){cn_CL<-cn_CL+1}

			for(nbc in 1:trunc(cn/2+1)){
				Centr_CL[4,cn_CL+nbc-1]<-max(0.5,min(1,max_F[trunc((cn+1)/2)+nbc]))
			}#nbc
		}#cn

		for(nbc in 1:trunc(cn/2+1)){
			Centr_CL[4,cn_CL+nbc-1]<-max(0.5,min(1,max_F[trunc((cn+1)/2)+nbc]))
		}

### calculating 0 copy level 

Centr_CL[5,1]<-Centr_CL[5,2]-1.5*(Centr_CL[5,4]-Centr_CL[5,2])

### if tumor purity is close to ~97% consider it to be 100%, as its hard to distinguish

if(Centr_CL[4,4]<0.98){
	for(cn_CL in 5:25){
		if(Centr_CL[2,cn_CL]==Centr_CL[1,cn_CL]){
			Centr_CL[4,cn_CL]<-min(0.97,Centr_CL[4,cn_CL])
		}
	}
}

out<-Centr_CL

}# function getCenterClusterGAP


#########################################################################
##################
################## Function GAPonNETcoordinates
##################
#########################################################################

GAPonNETcoordinates<-function(Centr_CL,SF,SR){

NF<-length(SF)
NR<-length(SR)
step_SF<-SF[2]-SF[1]
step_SR<-SR[2]-SR[1]

CN<-max(Centr_CL[1,])

NET_C<-matrix(0,CN+1+1,NF)

rownames(NET_C)<-seq(0,CN+1,1)
colnames(NET_C)<-seq(1,NF,1)

NET_C[,(dim(NET_C)[[2]]-3)]<-1

cn_CL<-1
for(cn_CL in 1:dim(Centr_CL)[[2]]){

	max_R<-min(abs(Centr_CL[5,cn_CL]-SR))
	if(max_R<step_SR){
		max_R<-which.min(abs(Centr_CL[5,cn_CL]-SR))
		if(Centr_CL[5,cn_CL]>=SR[max_R]){max_R<-min(NR,max_R+1)}
		}else{
	max_R<-round(max_R/step_SR)
	if(Centr_CL[5,cn_CL]>SR[NR]){max_R<-NR+max_R}
	if(Centr_CL[5,cn_CL]<SR[1]){max_R<-(-max_R)}
	}

	rownames(NET_C)[Centr_CL[1,cn_CL]+1]<-max_R
	max_F<-which.min(abs(SF-Centr_CL[4,cn_CL]))
	NET_C[Centr_CL[1,cn_CL]+1,max_F]<-cn_CL
	NET_C[Centr_CL[1,cn_CL]+1,(dim(NET_C)[[2]]-3)]<-Centr_CL[1,cn_CL]+30
	NET_C[dim(NET_C)[[1]],max_F]<-NET_C[dim(NET_C)[[1]],max_F]+1
}# cn_CL

tt<-which(NET_C[dim(NET_C)[[1]],]==0);if(length(tt)>0){NET_C<-NET_C[,-tt]}
NET_C<-NET_C[-dim(NET_C)[[1]],]

out<-NET_C

} # function GAPonNETcoordinates


#########################################################################
##################
################## Function GAPonNET_template
##################
#########################################################################

GAPonNET_template<-function(Centr_CL,NET_C,SF){

NF<-length(SF)
step_SF<-SF[2]-SF[1]


NET_tmp<-as.numeric(NET_C)

for(k in 1:4){NET_tmp<-cbind(NET_tmp,NET_tmp)}
NET_tmp<-NET_tmp[,1:9]
colnames(NET_tmp)<-c("N_LRR","N_BAF","Step_LRR_down","Step_LRR_up","Step_BAF_le","Step_BAF_ri","Copy","BAF","Clust_Num")

tt<-as.numeric(rownames(NET_C))
NET_tmp[,1]<-rep(tt,dim(NET_C)[[2]])
NET_tmp[,2]<-rep(as.numeric(colnames(NET_C)),each=dim(NET_C)[[1]])
NET_tmp[,3]<-rep(tt-1-c(tt[1]-3,tt[1:(length(tt)-1)]),dim(NET_C)[[2]])
NET_tmp[,4]<-rep(c(tt[2:length(tt)],tt[length(tt)]+3)-tt-1,dim(NET_C)[[2]])
tt<-which(NET_tmp[,9]==0); if(length(tt)>0){NET_tmp<-NET_tmp[-tt,]}

tt<-which(NET_tmp[,7]<30);if(length(tt)>0){NET_tmp[tt,7]<-Centr_CL[1,NET_tmp[tt,7]]}
tt<-which(NET_tmp[,7]>=30);if(length(tt)>0){NET_tmp[tt,7]<-NET_tmp[tt,7]-30}

tt<-which(NET_tmp[,8]<30);if(length(tt)>0){NET_tmp[tt,8]<-Centr_CL[2,NET_tmp[tt,8]]}
tt<-which(NET_tmp[,8]>=30);if(length(tt)>0){NET_tmp[tt,8]<-NET_tmp[tt,8]<-0}

NET_tmp<-NET_tmp[order(NET_tmp[,1],NET_tmp[,2],decreasing=T),]

NET_tmp[,3:4]<-round((NET_tmp[,3:4]-2)/2.5,0)

NET_tmp[1:(dim(NET_tmp)[[1]]-1),5]<-NET_tmp[1:(dim(NET_tmp)[[1]]-1),2]-NET_tmp[2:dim(NET_tmp)[[1]],2]-1

tt<-which(NET_tmp[,5]<0); if(length(tt)>0){NET_tmp[tt,5]<-(4/3)*(NET_tmp[tt,2]-1)}
NET_tmp[dim(NET_tmp)[[1]],5]<-NET_tmp[dim(NET_tmp)[[1]],2]-1

NET_tmp[2:dim(NET_tmp)[[1]],6]<-NET_tmp[1:(dim(NET_tmp)[[1]]-1),2]-NET_tmp[2:dim(NET_tmp)[[1]],2]-1

for(k in 1:dim(NET_tmp)[[1]]){
	if((NET_tmp[k,8]==0)&&(NET_tmp[k,5]>=7)){NET_tmp[k,5]<-16}
	if((NET_tmp[k,8]==NET_tmp[k,7])&&(abs(NET_tmp[k,6])<16)){NET_tmp[k,6]<-(97-NET_tmp[k,2])*4}
}

NET_tmp[,5:6]<-round(NET_tmp[,5:6]/4,0)

tt<-which(NET_tmp[,6]<0);if(length(tt)>0){NET_tmp[tt,6]<-NF-NET_tmp[tt,2]}
NET_tmp[1,6]<-max(1,NF-NET_tmp[1,2])

tt<-which(NET_tmp[,5]>4); if(length(tt)>0){NET_tmp[tt,5]<-4}

for(k in 1:dim(NET_tmp)[[1]]){
	if(NET_tmp[k,6]>4){
		if(NET_tmp[k,2]==1){
			NET_tmp[k,6]<-min(7,NET_tmp[k,6])
		}else{NET_tmp[k,6]<-4}
	}
}#k

out<-NET_tmp

}# function GAPonNET_template



#########################################################################
##################
################## Function markNETtemplate
##################
#########################################################################

markNETtemplate<-function(NET,NET_tmp){


NF<-dim(NET)[[2]]
NR<-dim(NET)[[1]]
SF<-as.numeric(colnames(NET))
SR<-as.numeric(rownames(NET))
step_SF<-SF[2]-SF[1]
step_SR<-SR[2]-SR[1]



NET1<-NET

NET1<-NET1*0
NET2<-NET1*0
NET3<-NET1*0

p<-1
for(p in 1:dim(NET_tmp)[[1]]){
	m<-max(1,(NET_tmp[p,1]-NET_tmp[p,3]))
	k<-min((NET_tmp[p,1]+NET_tmp[p,4]),NR)
	s<-(NET_tmp[p,2]-NET_tmp[p,5])
	n<-(NET_tmp[p,2]+NET_tmp[p,6])
	NET1[m:k,s:n]<-(-NET_tmp[p,9])
	NET3[m:k,s:n]<-(-NET_tmp[p,9])
	j<-50	
	while(j>=1){
		NET2[max(1,m-j):min(NR,k+j),max(1,s-j):min(NF,n+j)]<-j
		j<-j-1
	}	
	for(j in max(1,m-50):min(NR,k+50)){
		for(jj in max(1,s-50):min(NF,n+50)){
			if((NET1[j,jj]>0)&&(NET2[j,jj]<NET1[j,jj])||(NET1[j,jj]==0)){
				NET1[j,jj]<-NET2[j,jj]
				NET3[j,jj]<-NET_tmp[p,9]
			}
	}	}
}#p


for(j in 3:(dim(NET3)[[1]]-3)){
	for(p in 3:(dim(NET3)[[2]]-3-8)){
		tt<-as.numeric(abs(NET3[j:(j+1),p:(p+1)]))
		if(length(which(tt==0))>0){tt<-tt[-which(tt==0)]}
		if((length(tt)>0)&&(min(tt)!=max(tt))){
		NET3[(j-2):(j+3),(p-2):(p+3)]<-0
		}
}	}

for(j in 3:(dim(NET3)[[1]]-3)){
	for(p in (dim(NET3)[[2]]-3-8):(dim(NET3)[[2]]-1)){
		tt<-as.numeric(abs(NET3[j:(j+1),p:(p+1)]))
		if(length(which(tt==0))>0){tt<-tt[-which(tt==0)]}
		if((length(tt)>0)&&(min(tt)!=max(tt))){
			NET3[(j-2):(j+3),p:(p+1)]<-0
		}
}	}


for(j in 1:dim(NET3)[[1]]){
	for(p in 1:dim(NET3)[[2]]){
		if(NET3[j,p]>0){
			tt<-NET3[max(1,j-6):min(NR,j+6),max(1,p-6):min(NF,p+6)]+NET3[j,p]
			if(length(which(tt==0))==0){NET3[j,p]<-0}
		}
}	}

NET3[,(dim(NET3)[[2]]-2):dim(NET3)[[2]]]<-0

rm(NET2)

for(p in 1:dim(NET1)[[1]]){
	tt<-which(NET1[p,]>0);if(length(tt)>0){NET1[p,tt]<-0}
	tt<-which(NET3[p,]>=0);if(length(tt)>0){NET1[p,tt]<-0}
}

NET<-NET*(1+sign(NET1))+NET1

NETM<-as.numeric(NET)
NETM<-cbind(NETM,NETM,NETM)
NETM[,1]<-rep(seq(1,length(SR)),length(SF))
NETM[,2]<-rep(seq(1,length(SF)),each=length(SR))
NETM<-NETM[-which(NETM[,3]==0),]
NETM<-cbind(NETM,NETM[,3])
NETM<-NETM[order(NETM[,3],decreasing=F),]

###############  Deleting subclones closer than 3 cells to definite cluster

p<-1
while((p<=dim(NETM)[[1]])&&(NETM[p,3]<0)){p<-p+1}

if(p<dim(NETM)[[1]]){NETM[1:(p-1),4]<-4}

for(k in 1:(p-1)){
	if(p<=dim(NETM)[[1]]){
		NETM[p:dim(NETM)[[1]],4]<-abs(NETM[p:dim(NETM)[[1]],1]-NETM[k,1])
		tt<-which(NETM[p:dim(NETM)[[1]],4]<4)
		if(length(tt)>0){
	
			NETM[(p-1+tt),4]<-abs(NETM[(p-1+tt),2]-NETM[k,2])
			tt<-which(NETM[p:dim(NETM)[[1]],4]<4)
			if(length(tt)>0){NETM<-NETM[-(p-1+tt),]}
		}
	}
}#k	



NETM[,4]<-NETM[,3]

####################################
###############  Enumerating subclones 

NETM[,3]<-NETM[,4]

NETM<-NETM[order(NETM[,3],decreasing=T),]

p<-1
while(NETM[p,3]>0){p<-p+1}

if(p>1){

	Dist<-as.matrix(dist(NETM[1:(p-1),1:2],method="maximum",diag=T,upper=T))
	Dist[which(Dist>1)]<-0
	for(k in 1:dim(Dist)[[1]]){Dist[k,k]<-1}


	cn_CL<-(-40)

	for(k in 1:dim(Dist)[[1]]){
		if(NETM[k,3]>0){
			NETM[which(Dist[,k]!=0),3]<-cn_CL

			if((k+1)<=dim(Dist)[[1]]){
				for(m in (k+1):dim(Dist)[[1]]){
					if(sum(Dist[,k]*Dist[,m])>0){
						Dist[,k]<-Dist[,k]+Dist[,m]
						NETM[which(Dist[,k]!=0),3]<-cn_CL
					}
				}#m
			}
			cn_CL<-cn_CL-1}
		}
	}

	NET<-NET*0
	for(k in 1:dim(NETM)[[1]]){
	NET[NETM[k,1],NETM[k,2]]<-NETM[k,3]
}# if p>1

NET<-abs(NET)

NETM<-abs(NETM)
NETM[,1]<-SR[NETM[,1]]
NETM[,2]<-SF[NETM[,2]]
NETM<-NETM[order(NETM[,3],decreasing=F),]

out<-NETM

}# function markNETtemplate



#########################################################################
##################
################## Function GetGAPtemplateZones
##################
#########################################################################


GetGAPtemplateZones<-function(NET,Centr_CL) {


NF<-dim(NET)[[2]]
NR<-dim(NET)[[1]]
SF<-as.numeric(colnames(NET))
SR<-as.numeric(rownames(NET))
step_SF<-SF[2]-SF[1]
step_SR<-SR[2]-SR[1]


##### Representing GAP in terms of NET coordinates: matrix NET_C

NET_C<-GAPonNETcoordinates(Centr_CL,SF,SR)


##### Representing GAP by template description on NET: matrix NET_tmp

NET_tmp<-GAPonNET_template(Centr_CL,NET_C,SF)

##### Comming back to NET, adding rows (enlarging LRR range) if necessary 

p<-1
while((p<=dim(NET_tmp)[[1]])&&(NET_tmp[p,1]>dim(NET)[[1]])){p<-p+1}

if((p>1)&&(p<=dim(NET_tmp)[[1]])){
	tt<-matrix(0,NET_tmp[p-1,1]-NET_tmp[p,1]+1,dim(NET)[[2]])
	rownames(tt)<-seq(SR[NR]+step_SR,SR[NR]+dim(tt)[[1]]*step_SR,step_SR)
	NET<-rbind(NET,tt)
}

while((p<=dim(NET_tmp)[[1]])&&(NET_tmp[p,1]>0)){p<-p+1}

if(p<dim(NET_tmp)[[1]]){
	tt<-matrix(0,(-NET_tmp[p,1]+1),dim(NET)[[2]])
	rownames(tt)<-seq(SR[1]-step_SR*dim(tt)[[1]],SR[1]-step_SR,step_SR)
	NET<-rbind(tt,NET)
	NET_tmp[,1]<-NET_tmp[,1]-NET_tmp[p,1]+1
}

##### Correcting NET_tmp range if necessary (if it exceeds the NET range) 

tt<-which(NET_tmp[,1]<0);if(length(tt)>0){NET_tmp<-NET_tmp[-tt,]}
tt<-which(NET_tmp[,1]>dim(NET)[[1]]); if(length(tt)>0){NET_tmp<-NET_tmp[-tt,]}

SR<-as.numeric(rownames(NET))
NR<-length(SR)


out<-markNETtemplate(NET,NET_tmp)

}#function GetGAPtemplateZones





#########################################################################
##################
################## Function subClonalClusters
##################
#########################################################################


subClonalClusters<-function(NETM,Centr_CL){

CN_CL<-rbind(Centr_CL[1,],Centr_CL[5,])
CN_CL<-table(CN_CL[1,],CN_CL[2,])
CN_CL<-rbind(as.numeric(rownames(CN_CL)),as.numeric(colnames(CN_CL)))

BAF_CL<-rbind(Centr_CL[1,],Centr_CL[2,],Centr_CL[4,])
BAF_CL<-BAF_CL[,order(BAF_CL[3,],decreasing=F)]

SB_CL<-numeric(0)

if(length(which(NETM[,3]>=40))>1){
if(length(which(NETM[,3]<40))>0){
	SB_CL<-NETM[-which(NETM[,3]<40),]
	SB_CL<-SB_CL[order(SB_CL[,3],decreasing=F),]

	SB_CL[,1]<-SB_CL[,1]*SB_CL[,4]
	SB_CL[,2]<-SB_CL[,2]*SB_CL[,4]

	tt<-SB_CL[1,]
	k<-4
	for(k in 2:dim(SB_CL)[[1]]){
		if(SB_CL[k,3]==tt[3]){
			SB_CL[k,1]<-SB_CL[k,1]+SB_CL[k-1,1]
			SB_CL[k,2]<-SB_CL[k,2]+SB_CL[k-1,2]
			SB_CL[k,4]<-SB_CL[k,4]+SB_CL[k-1,4]
			SB_CL[k-1,2]<-0
		}else{
			SB_CL[k-1,1]<-SB_CL[k-1,1]/SB_CL[k-1,4]
			SB_CL[k-1,2]<-SB_CL[k-1,2]/SB_CL[k-1,4]

			tt<-SB_CL[k,]
			}
	}#k

	SB_CL[k,1]<-SB_CL[k,1]/SB_CL[k,4]
	SB_CL[k,2]<-SB_CL[k,2]/SB_CL[k,4]


	if(length(which(SB_CL[,2]==0))>0){
		if(length(which(SB_CL[,2]!=0))==1){
			SB_CL<-rbind(SB_CL,cbind(SB_CL[,1:2],SB_CL[,3:4]+1))
		}
		SB_CL<-SB_CL[-which(SB_CL[,2]==0),]
		SB_CL<-SB_CL[order(SB_CL[,3],decreasing=F),]
	}
	
	SB_CL<-cbind(SB_CL[,3],SB_CL[,2:4],SB_CL[,1:2])
	SB_CL[,2:4]<-0
	SB_CL<-t(SB_CL)
	rownames(SB_CL)<-c("Subclone","CN_LRR","B_All","CN_BAF","SB_LRR","SB_BAF")

	for(k in 1:dim(SB_CL)[[2]]){

			p<-1
			while((p<=dim(CN_CL)[[2]])&&(SB_CL[5,k]>CN_CL[2,p])){p<-p+1}
			if(p>dim(CN_CL)[[2]]){
				SB_CL[2,k]<-CN_CL[1,p-1]
			}else{
				if(p==1){
					SB_CL[2,k]<-CN_CL[1,p]
				}else{
					w<-c((CN_CL[2,p]-SB_CL[5,k]),(SB_CL[5,k]-CN_CL[2,p-1]))
					w<-w/(CN_CL[2,p]-CN_CL[2,p-1])
					SB_CL[2,k]<-weighted.mean(c(CN_CL[1,p-1],CN_CL[1,p]),w)}
					}

				SB_CL[2,k]<-round(SB_CL[2,k],1)
				if((p>1)&&(SB_CL[2,k]==CN_CL[1,p-1])){SB_CL[2,k]<-SB_CL[2,k]+0.1}
				if((p<=dim(CN_CL)[[2]])&&(SB_CL[2,k]==CN_CL[1,p])){SB_CL[2,k]<-SB_CL[2,k]-0.1}

				if(SB_CL[6,k]>=0.97){
					SB_CL[3,k]<-SB_CL[2,k]
					SB_CL[4,k]<-SB_CL[2,k]
				}else{
					if((p>1)&&(length(which(BAF_CL[1,]<CN_CL[1,p-1]))>0)){
						tmp_BAF_CL<-BAF_CL[,-which(BAF_CL[1,]<CN_CL[1,p-1])]
					}else{
						tmp_BAF_CL<-BAF_CL
						}
					if(p<=dim(CN_CL)[[2]]){
						tt<-which(tmp_BAF_CL[1,]>CN_CL[1,p])
						if(length(tt)>0){
							if(length(tt)==(dim(tmp_BAF_CL)[[2]]-1)){tmp_BAF_CL<-cbind(tmp_BAF_CL,tmp_BAF_CL)}
							tmp_BAF_CL<-tmp_BAF_CL[,-which(tmp_BAF_CL[1,]>CN_CL[1,p])]}
						}

					tmp_BAF_CL<-cbind(tmp_BAF_CL,tmp_BAF_CL[,dim(tmp_BAF_CL)[[2]]])
					tmp_BAF_CL[3,dim(tmp_BAF_CL)[[2]]]<-1

					p<-1
					while((p<dim(tmp_BAF_CL)[[2]])&&(SB_CL[6,k]>tmp_BAF_CL[3,p])){p<-p+1}
					if(p==1){p<-p+1}

					w<-c((tmp_BAF_CL[3,p]-SB_CL[6,k]),(SB_CL[6,k]-tmp_BAF_CL[3,p-1]))
					w<-w/(tmp_BAF_CL[3,p]-tmp_BAF_CL[3,p-1])
					if(tmp_BAF_CL[1,p-1]!=0){
						SB_CL[3,k]<-weighted.mean(c(tmp_BAF_CL[2,p-1]/tmp_BAF_CL[1,p-1],tmp_BAF_CL[2,p]/tmp_BAF_CL[1,p]),w)
					}else{
						SB_CL[3,k]<-weighted.mean(c(tmp_BAF_CL[2,p-1],tmp_BAF_CL[2,p]),w)
						}
					SB_CL[3,k]<-round(SB_CL[3,k]*SB_CL[2,k],1)


					if(p!=dim(tmp_BAF_CL)[[2]]){
						SB_CL[4,k]<-weighted.mean(c(tmp_BAF_CL[1,p-1],tmp_BAF_CL[1,p]),w)
						SB_CL[4,k]<-round(SB_CL[4,k],1)
						if(SB_CL[4,k]==tmp_BAF_CL[1,p-1]){SB_CL[4,k]<-SB_CL[4,k]+0.1}
						if(SB_CL[4,k]==tmp_BAF_CL[1,p]){SB_CL[4,k]<-SB_CL[4,k]-0.1}
					}else{}

				} # else for BAF>0.97


		}#k

}}# if there are subclones

out<-SB_CL
}# function subClonalClusters





#########################################################################
##################
################## Function CopyNumber_and_Genotype
##################
#########################################################################
#seg_LRR<-tmp[notNAs,c_lrr]
#seg_BAF<-tmp[notNAs,c_baf]
#seg_Len<-tmp[notNAs,c_len]
#showGAPTemplate=T

CopyNumber_and_Genotype<-function (seg_LRR,seg_BAF,seg_Len,showGAPTemplate){

#######################################################
####  1 Data preparation: determination of net, density, etc.

tmp<-matrix(0,length(seg_LRR),6)
colnames(tmp)<-c("Index","LRR","BAF","Len","sup_L","sup_B")

tmp[,1]<-seq(1:length(seg_LRR))
tmp[,2:4]<-cbind(seg_LRR,seg_BAF,seg_Len)

C_ind<-1;C_lrr<-2;C_baf<-3;C_len<-4;C_nL<-5;C_nB<-6;

##############################"
####  SR,SF vector of discretized range of LRR and BAF correspondignly
####  NR=length(SR),NF<-length(SF)
####  NET is matrix BAFxLRR with discretized density of GAP


R_min<-round((min(tmp[which(tmp[,C_len]>10),C_lrr],na.rm=T)-0.15)*10,0)/10
R_max<-round((max(tmp[which(tmp[,C_len]>10),C_lrr],na.rm=T)+0.15)*10,0)/10

step_SR<-0.005
step_SF<-0.005

NR<-round(((R_max-R_min)/step_SR),0)
SR<-seq((R_min+step_SR),R_max,step_SR)

NF<-round(((1.02+step_SF-0.5)/step_SF),0)
SF<-seq(0.5+step_SF,1.02+step_SF,step_SF)

NET<-matrix(0,NR,NF)

rownames(NET)<-SR
colnames(NET)<-SF



#### Determining coordinates of regions on NET
k<-1
for(k in 1:dim(tmp)[[1]]){
	tmp[k,C_nL]<-which.min(abs(SR-tmp[k,C_lrr]))
	if(tmp[k,C_lrr]>=SR[tmp[k,C_nL]]){tmp[k,C_nL]<-min(tmp[k,C_nL]+1,NR)}
	tmp[k,C_nB]<-which.min(abs(SF-tmp[k,C_baf]))
	if(tmp[k,C_baf]>=SF[tmp[k,C_nB]]){tmp[k,C_nB]<-min(tmp[k,C_nB]+1,NF)}
	
	if(tmp[k,C_len]>=50){
		NET[tmp[k,C_nL],tmp[k,C_nB]]<-NET[tmp[k,C_nL],tmp[k,C_nB]]+tmp[k,C_len]
	}
}

#########################################
#### Smoothing density on NET: simple averaging density of adjucent 9 cells
#### kk weight of the center cell, sometime its worth to enhance kk for clarifying the picture;
#### however, almost always it works nice with kk=1


NET<-smoothNET(NET,weightC=1)



################################################
#### Clarifying the structure by filtering low density regions
#### The threshold for low density is 50 (set semi arbitrarilly 
#### to avoid intersection with cluster numbering: 1-40)
####


NET[which(NET<100)]<-0
plotGAPdensity(NET)

#tt<-(quantile(LRR,prob=0.95,na.rm=T)-quantile(LRR,prob=0.05,na.rm=T))/0.005*5
#tt<-min(0.99,(1-tt/(dim(NET)[[1]]*dim(NET)[[2]])))
#tt<-quantile(NET,prob=tt,na.rm=T)
#tt<-quantile(NET,prob=0.985,na.rm=T)
#NET[which(NET<tt)]<-0


##################################################################
####
#### 2 Determination of GAP pattern
####
###################################################################


FF_all<-fittingGAP(NET)

######## FF_all contains several top solutions for GAP
######## kk by default is the first one, however, sometime for comlicated pattern the right one is 2,3..

kk<-as.numeric(rownames(table(FF_all[9,])))[1]

#if(showGAPTemplate){
#tt<-plotGAPdensity(NET)
#for(k in 1:dim(FF_all)[[2]]){
#	if(FF_all[9,k]==kk){
#		Delta<-which(FF_all[5:7,k]!=0)
#		Delta<-c(Delta[1]+1, SR[FF_all[Delta[1]+4,k]])
#		p_BAF<-FF_all[2,k]
#		q_LRR<-FF_all[3,k]
#		Centr_CL<-getCenterClusterGAP(CN,p_BAF,q_LRR,Delta)
#		for(j in 1:dim(Centr_CL)[[2]]){
#			text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""),cex=1)
#		}
#	}#if right Set_pattern
#}#k
#}# if show GAP template

################### Final decision
### p_BAF and q_LRR are set to the mean of final pattern set shown above

tt<-which(FF_all[9,]!=kk)

if(length(tt)>0){
	if(length(tt)==(dim(FF_all)[[2]]-1)){FF_final<-cbind(FF_all,FF_all)
		}else{FF_final<-FF_all}
	FF_final<-FF_final[,-which(FF_final[9,]!=kk)]
}

p_BAF<-round(mean(FF_final[2,]),2)
q_LRR<-round(mean(FF_final[3,]),2)

for(k in 4:8){
	if(sum(FF_final[k,])>0){FF_final[k,1]<-round(mean(FF_final[k,which(FF_final[k,]!=0)]),0)}
}#k

FF_final[4:8,1]<-round(FF_final[4:8,1],0)

Delta<-which(FF_final[5:7,1]!=0)
Delta<-c(Delta[1]+1, round(SR[FF_final[Delta[1]+4,1]],2))
		
############ GAP structure
Centr_CL<-getCenterClusterGAP(CN,p_BAF,q_LRR,Delta)

############ Marking cluster zones
NETM<-GetGAPtemplateZones(NET,Centr_CL)

############ Summarizing Subclones
SB_CL<-subClonalClusters(NETM,Centr_CL)

########## Adjusting NETM to have ALL clusters
for(k in 1:dim(Centr_CL)[[2]]){
tt<-which(NETM[,3]==Centr_CL[3,k])
if(length(tt)==0){NETM<-rbind(NETM,c(Centr_CL[5,k],Centr_CL[4,k],Centr_CL[3,k],Centr_CL[3,k]))}
}
NETM<-NETM[order(NETM[,3],decreasing=F),]


############ GAP figure with recognition pattern

if(showGAPTemplate){

tt<-plotGAPdensity(NET)

#RB<-rainbow(40)
#for(p in 1:38){
#	tt1<-which(NETM[,3]==p)
#	if(length(tt1)>0){
#		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
#	}
#}

for(j in 1:dim(Centr_CL)[[2]]){
	text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""),cex=1)
}

text(0.6,-0.5,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=1)
text(0.6,-0.7,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=1)
text(0.6,-0.9,paste("2copy = ",as.character(round(Centr_CL[5,4],2)),sep=""),cex=1)

}# if show GAP template

out<-list(NETM=NETM, Centr_CL=Centr_CL, SB_CL=SB_CL, coeff=c(p_BAF,q_LRR))

#out

}# function CopyNumber_and_Genotype




#########################################################################
##################
################## Function CopyNumber_and_Genotype
##################
#########################################################################


CopyNumber_and_GenotypeWOfitting<-function (seg_LRR,seg_BAF,seg_Len,showGAPTemplate,p_BAF,q_LRR,Delta){


tmp<-matrix(0,length(seg_LRR),6)
tmp[,1]<-seq(1:length(seg_LRR))
tmp[,2:4]<-cbind(seg_LRR,seg_BAF,seg_Len)
C_ind<-1;C_lrr<-2;C_baf<-3;C_len<-4;C_nL<-5;C_nB<-6;



#######################################################
####  Determination of net, density, etc.
####  SR,SF vector of discretized range of LRR and BAF correspondignly
####  NR=length(SR),NF<-length(SF)
####  NET is matrix BAFxLRR with discretized density of GAP


R_min<-round((min(tmp[which(tmp[,C_len]>10),C_lrr],na.rm=T)-0.15)*10,0)/10
R_max<-round((max(tmp[which(tmp[,C_len]>10),C_lrr],na.rm=T)+0.15)*10,0)/10

step_SR<-0.005
step_SF<-0.005

NR<-round(((R_max-R_min)/step_SR),0)
SR<-seq((R_min+step_SR),R_max,step_SR)

NF<-round(((1.02+step_SF-0.5)/step_SF),0)
SF<-seq(0.5+step_SF,1.02+step_SF,step_SF)

NET<-matrix(0,NR,NF)

rownames(NET)<-SR
colnames(NET)<-SF

#### Determining coordinates of regions on NET
k<-1
for(k in 1:dim(tmp)[[1]]){
	tmp[k,C_nL]<-which.min(abs(SR-tmp[k,C_lrr]))
	if(tmp[k,C_lrr]>=SR[tmp[k,C_nL]]){tmp[k,C_nL]<-min(tmp[k,C_nL]+1,NR)}
	tmp[k,C_nB]<-which.min(abs(SF-tmp[k,C_baf]))
	if(tmp[k,C_baf]>=SF[tmp[k,C_nB]]){tmp[k,C_nB]<-min(tmp[k,C_nB]+1,NF)}
	
	if(tmp[k,C_len]>=10){
		NET[tmp[k,C_nL],tmp[k,C_nB]]<-NET[tmp[k,C_nL],tmp[k,C_nB]]+tmp[k,C_len]
	}
}

###### Smoothing and contrasting NET

NET<-smoothNET(NET,weightC=1)
NET[which(NET<50)]<-0


##############################################################
#####
###### GAP structure construction
#####
##############################################################

Centr_CL<-getCenterClusterGAP(CN,p_BAF,q_LRR,Delta)

NETM<-GetGAPtemplateZones(NET,Centr_CL)

SB_CL<-subClonalClusters(NETM,Centr_CL)

########## Adjusting NETM to have ALL clusters
for(k in 1:dim(Centr_CL)[[2]]){
tt<-which(NETM[,3]==Centr_CL[3,k])
if(length(tt)==0){NETM<-rbind(NETM,c(Centr_CL[5,k],Centr_CL[4,k],Centr_CL[3,k],Centr_CL[3,k]))}
}
NETM<-NETM[order(NETM[,3],decreasing=F),]



############## GAP figure with recognition pattern

if(showGAPTemplate){

tt<-plotGAPdensity(NET)

for(j in 1:dim(Centr_CL)[[2]]){
	text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""),cex=1)
}
text(0.6,-0.5,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=1)
text(0.6,-0.7,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=1)
text(0.6,-0.9,paste("2copy = ",as.character(round(Centr_CL[5,4],2)),sep=""),cex=1)
}# if show GAP template

out<- list(NETM=NETM, Centr_CL=Centr_CL, SB_CL=SB_CL, coeff=c(p_BAF,q_LRR))

#out

}# function CopyNumber_and_Genotype



#########################################################################
##################
################## Function get ranking of possible Copy Numbers based on CN value
##################
##################
#######################################################

#cnV<-(-1.5)

getCNrank<-function(cnV,c_CL){

	c_CL[4,]<-c_CL[2,]-cnV
	kk<-which.min(abs(c_CL[4,]))

	c_CL[5,kk]<-10
	if(kk==1){if((c_CL[4,kk]<0)&&(c_CL[3,kk+1]<abs(c_CL[4,kk]))){c_CL[5,kk+1]<-9}else{c_CL[5,kk+1]<-8}
	}else{if(kk==9){if((c_CL[4,kk]>0)&&(c_CL[3,kk]<c_CL[4,kk])){c_CL[5,kk-1]<-9}else{c_CL[5,kk-1]<-8}

		}else{if(c_CL[4,kk]>=0){if(c_CL[3,kk]<c_CL[4,kk]){c_CL[5,kk-1]<-9;c_CL[5,kk+1]<-8
						}else{c_CL[5,kk-1]<-8;c_CL[5,kk+1]<-8}
			}else{if(c_CL[3,kk+1]<abs(c_CL[4,kk])){c_CL[5,kk+1]<-9;c_CL[5,kk-1]<-8
				}else{c_CL[5,kk+1]<-8;c_CL[5,kk-1]<-8}

				}
			}
		}
	kk<-which(c_CL[5,]==1)
	c_CL[5,kk]<-order(abs(c_CL[4,kk]),decreasing=T)+1

	out<-c_CL[5,]
	out
}



#########################################################################
##################
################## Function getCNwithPrecision
##################
#########################################################################

#Precision<-1/2

getCNwithPrecision<-function(vectCN,Centr_CL,Precision){

ss<-which(!is.na(vectCN))
vectCN_init<-vectCN
vectCN<-vectCN_init[ss]

if(length(vectCN)>0){
c_CL<-table(Centr_CL[5,])
c_CL<-rbind(seq(0,8),as.numeric(rownames(c_CL)))
c_CL<-rbind(c_CL,c_CL,seq(0,8))
c_CL[3,2:dim(c_CL)[[2]]]<-c_CL[2,2:dim(c_CL)[[2]]]-c_CL[2,1:(dim(c_CL)[[2]]-1)]
c_CL[3,1]<-1
c_CL[3,]<-round(c_CL[3,]*Precision,4)
c_CL[5,]<-1	

Centr_CL<-cbind(Centr_CL,Centr_CL[,which(Centr_CL[1,]-Centr_CL[2,]==0)])
Centr_CL[2,26:34]<-0
Centr_CL[3,26:34]<-Centr_CL[1,26:34]+30
Centr_CL[4,26:34]<-1
Centr_CL<-Centr_CL[,order(Centr_CL[3,],decreasing=F)]

vectRang<-vectCN*0

k<-3
for(k in 1:length(vectCN)){

	c_CL[5,]<-1
	c_CL[5,]<-getCNrank(vectCN[k],c_CL)
	tt<-which(c_CL[5,]==10)
	tt1<-which(c_CL[5,]==9)
	if(length(tt1)==1){
		kk<-round(abs(c_CL[4,tt]-vectCN[k])/abs(c_CL[4,tt]-c_CL[4,tt1]),1)
		vectRang[k]<-c_CL[1,tt]*(1-kk)+c_CL[1,tt1]*kk
	}else{vectRang[k]<-c_CL[1,tt]}

}#

vectCN_init[ss]<-vectRang
}

out<-vectCN_init

}#function


#########################################################################
##################
################## Function getGTwithRank
##################
#########################################################################


getGTwithRank<-function(tmp,Centr_CL,NETM){


Centr_CL<-cbind(Centr_CL,Centr_CL[,which(Centr_CL[1,]-Centr_CL[2,]==0)])
Centr_CL[2,26:34]<-0
Centr_CL[3,26:34]<-Centr_CL[1,26:34]+30
Centr_CL[4,26:34]<-1
Centr_CL<-Centr_CL[,order(Centr_CL[3,],decreasing=F)]

weigth_BAF<-round((Centr_CL[4,5]-Centr_CL[4,3])/(Centr_CL[5,5]-Centr_CL[5,3]),1)
weigth_BAF<-max(min(weigth_BAF,4),1)

tmp[,c_nS]<-0

k<-395
for(k in 1:dim(tmp)[[1]]){

	if(!is.na(tmp[k,c_lrr]+tmp[k,c_baf])){

	if(trunc(tmp[k,c_nC])==tmp[k,c_nC]){kk<-c(tmp[k,c_nC])
	}else{kk<-c(trunc(tmp[k,c_nC]),trunc(tmp[k,c_nC])+1)}

	tt<-which(kk<0);if(length(tt)>0){kk<-kk[-tt]}
	tt<-which(kk>8);if(length(tt)>0){kk<-kk[-tt]}
	if(length(kk)==1){kk<-c(kk,kk)}

	kk<-Centr_CL[3,which(Centr_CL[1,]%in% kk)]
	if(length(kk)==1){kk<-c(kk,kk)}

	tt<-which(NETM[,3] %in% kk)
	if(length(tt)>1){tt<-NETM[tt,]}else{if(length(tt)==1){tt<-rbind(NETM[tt,],NETM[tt,])}}
	kk<-which(NETM[,3]>=40)
	if(length(kk)>0){tt<-rbind(tt,NETM[kk,])}

	tt[,4]<-sqrt((tt[,1]-tmp[k,c_lrr])^2+weigth_BAF*(tt[,2]-tmp[k,c_baf])^2)

	tt<-tt[order(tt[,3],tt[,4],decreasing=F),]
	hh<-table(tt[,3])
	hh<-c(0,hh)

	for(p in 2:length(hh)){
		tt[(hh[p-1]+1),4]<-sum(tt[(hh[p-1]+1):min((hh[p-1]+10),(hh[p-1]+hh[p])),4])/min(10,hh[p])
		if(1<hh[p]){tt[(hh[p-1]+2):(hh[p-1]+hh[p]),3]<-0}
			hh[p]<-hh[p-1]+hh[p]
		}

	ss<-which(tt[,3]==0)
	if(length(ss)>0){
		if(dim(tt)[[1]]-length(ss)>1){tt<-tt[-ss,]
		}else{tt<-tt[-ss,];tt<-rbind(tt,tt)}
	}

	tt<-tt[order(tt[,4],decreasing=F),]
	tt[,4]<-signif(tt[,4],digits=1)

	if((tt[1,3]>=40)&&(tmp[k,c_len]>=50)){tmp[k,c_nS]<-tt[1,3]}
	pp<-which(tt[,3]>=40)
	if(length(pp)>0){
		if(length(pp)<(dim(tt)[[1]]-2)){tt<-tt[-which(tt[,3]>=40),]
		}else{tt<-tt[-which(tt[,3]>=40),];tt<-rbind(tt,tt,tt)}
	}

	for(p in 1:min(3,dim(tt)[[1]])){
		if(tt[p,3]>=30){
			tmp[k,c_CN1+3*(p-1)]<-tt[p,3]-30
			tmp[k,c_BA1+3*(p-1)]<-0
		}else{
			tmp[k,c_CN1+3*(p-1)]<-Centr_CL[1,tt[p,3]]
			tmp[k,c_BA1+3*(p-1)]<-Centr_CL[2,tt[p,3]]
		}
		tmp[k,c_rnk1+3*(p-1)]<-tt[p,4]
	}#p

}
}#k


out<-tmp
}# function getCNandGTwithRank




###########################################################
####
####   Function UniteSEGtmp_one Uniting segments with similar attribution
####
###########################################################

UniteSEGtmp_one<-function(tmp){

FL<-T
while(FL==T){
FL<-F

for(k in 1:(dim(tmp)[[1]]-1)){

	if((tmp[k,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k,c_BA]==tmp[k+1,c_BA])){

		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_lrr]<-weighted.mean(c(tmp[k+1,c_lrr],tmp[k,c_lrr]),c(tmp[k+1,c_len],tmp[k,c_len]))
		tmp[k+1,c_nC]<-tmp[k+1,c_CN]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
		FL<-T
	}else{

	if((tmp[k,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_nC]==0)&&(tmp[k+1,c_nC]==0)){

		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_lrr]<-weighted.mean(c(tmp[k+1,c_lrr],tmp[k,c_lrr]),c(tmp[k+1,c_len],tmp[k,c_len]))
		tmp[k+1,c_nC]<-tmp[k+1,c_CN]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
		FL<-T
	}
	}
}#k

if(length(which(tmp[,1]==0))!=0){
tmp<-tmp[-which(tmp[,1]==0),]}

} #FL

out<-tmp

}# function





###########################################################
####
####   Function UniteSEGtmp_Illum Uniting segments with similar attribution
####
###########################################################

UniteSEGtmp_Illum<-function(tmp,p_BAF){


tmp<-UniteSEGtmp_one(tmp)

if(p_BAF>0.05){

FL<-T
while(FL==T){
FL<-F

for(k in 2:(dim(tmp)[[1]]-1)){

	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])
		&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_BA]==tmp[k+1,c_BA])&&(tmp[k+1,c_BA]!=0)
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k,c_BA]==0)&&(tmp[k,c_len]<100)){
		tmp[k,c_BA]<-tmp[k+1,c_BA]
		FL<-T
	}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_BA]==0)&&(tmp[k,c_len]<100)){
		if((tmp[k,c_chr]==tmp[k+1,c_chr])
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k+1,c_BA]!=0)
		&&((tmp[k-1,c_CN]!=tmp[k,c_CN])||(tmp[k-1,c_chr]!=tmp[k,c_chr]))){

		tmp[k,c_BA]<-tmp[k+1,c_BA]
		FL<-T
		}

		if((tmp[k,c_chr]==tmp[k-1,c_chr])
		&&(tmp[k,c_CN]==tmp[k-1,c_CN])&&(tmp[k-1,c_BA]!=0)
		&&((tmp[k+1,c_CN]!=tmp[k,c_CN])||(tmp[k+1,c_chr]!=tmp[k,c_chr]))){

		tmp[k,c_BA]<-tmp[k-1,c_BA]
		FL<-T
		}
	}

}#k

	if((tmp[1,c_BA]==0)&&(tmp[k,c_len]<100)){
		if((tmp[1,c_chr]==tmp[2,c_chr])&&(tmp[1,c_CN]==tmp[2,c_CN])&&(tmp[2,c_BA]!=0)){
			tmp[1,c_BA]<-tmp[2,c_BA]
			FL<-T
		}
	}
	k<-dim(tmp)[[1]]
	if((tmp[k,c_BA]==0)&&(tmp[k,c_len]<100)){
		if((tmp[k,c_chr]==tmp[k-1,c_chr])&&(tmp[k,c_CN]==tmp[k-1,c_CN])&&(tmp[k-1,c_BA]!=0)){
			tmp[k,c_BA]<-tmp[k-1,c_BA]
			FL<-T
		}
	}


tmp<-UniteSEGtmp_one(tmp)


} #FL

}# if not pure tumor


out<-tmp

}# function





###########################################################
####
####   Function adjustTMPstableSubcloneCN regions with intermediate CN fitted to exact neihbours
####
###########################################################

#tmp<-TMP

adjustTMPstableSubclone<-function(tmp,Centr_CL,NETM,p_BAF){

tt<-which(trunc(tmp[,c_nC])-tmp[,c_nC]!=0)

if(length(tt)>0){
	for(j in 1:length(tt)){
		if(tmp[tt[j],c_len]>500){tmp[tt[j],c_nC]<-tmp[tt[j],c_CN]}
		if(tt[j]==dim(tmp)[[1]]){if(tmp[tt[j],c_chr]!=tmp[tt[j]-1,c_chr]){tmp[tt[j],c_nC]<-tmp[tt[j],c_CN]}
		}else{
			if(tt[j]==1){if(tmp[tt[j],c_chr]!=tmp[tt[j]+1,c_chr]){tmp[tt[j],c_nC]<-tmp[tt[j],c_CN]}
			}else{
				if((tmp[tt[j],c_chr]!=tmp[tt[j]+1,c_chr])&&(tmp[tt[j],c_chr]!=tmp[tt[j]-1,c_chr])){tmp[tt[j],c_nC]<-tmp[tt[j],c_CN]}
				}
			}
	}#j
}

tt<-which(trunc(tmp[,c_nC])-tmp[,c_nC]!=0)

if(length(tt)>0){

	tmpB<-tmp[-tt,]
	if(length(tt)==1){tmpS<-rbind(tmp[tt,],tmp[tt,])}else{tmpS<-tmp[tt,]}

	tmpB<-UniteSEGtmp_Illum(tmpB,p_BAF)
	tmpS[,c_bafH]<-0
	j<-1
	for(j in 1:dim(tmpS)[[1]]){
		tt<-which(tmpB[,c_is]<tmpS[j,c_is])
		if(length(tt)>0){tt<-tt[length(tt)]
			if(tmpB[tt,c_if]>tmpS[j,c_if]){tmpS[j,c_bafH]<-tt
			}else{if(tt==dim(tmpB)[[1]]){tmpS[j,c_bafH]<-tt}else{tmpS[j,c_bafH]<-tt+0.5}}
		}else{tt<-0;tmpS[j,c_bafH]<-tt+0.5}
	}#j

	tt<-table(tmpS[,c_bafH])
	
	if(length(tt)==1){tt<-c(tt,tt);tt<-rbind(as.numeric(names(tt)),tt);ONE<-T
	}else{tt<-rbind(as.numeric(rownames(tt)),tt);ONE<-F}
	
	colnames(tt)<-seq(1,dim(tt)[[2]])
	
	TT<-1
	j<-1
	if(ONE){dim_tt<-1}else{dim_tt<-dim(tt)[[2]]}
	for(j in 1:dim_tt){
		if(trunc(tt[1,j])==tt[1,j]){

			if(tt[2,j]==1){
				if(abs(tmpS[TT,c_nC]-tmpB[tt[1,j],c_CN])<0.9){tmpS[TT,c_nC]<-tmpB[tt[1,j],c_CN]
				}else{tmpS[TT,c_nC]<-tmpS[TT,c_CN]}
			}else{
				if(max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[tt[1,j],c_CN]))<0.9){
					tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpB[tt[1,j],c_CN]
				}else{tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpS[TT:(TT+tt[2,j]-1),c_CN]}
				}
			TT<-TT+tt[2,j]
		}else{ kk<-trunc(tt[1,j])
			if(kk==0){kk<-1

			if(tt[2,j]==1){
				if(abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])<0.9){tmpS[TT,c_nC]<-tmpB[kk,c_CN]
				}else{tmpS[TT,c_nC]<-tmpS[TT,c_CN]}
			}else{
				if(max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk,c_CN]))<0.9){
						tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpB[kk,c_CN]
				}else{tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpS[TT:(TT+tt[2,j]-1),c_CN]}
				
				}
			TT<-TT+tt[2,j]

			}else{
			if(trunc(tmpB[kk,c_chr])==trunc(tmpB[kk+1,c_chr])){

			if(tt[2,j]==1){
				kk<-which(abs(tmpS[TT,c_nC]-tmpB[kk:(kk+1),c_CN])<0.9)
				if(length(kk)==0){tmpS[TT,c_nC]<-tmpS[TT,c_CN]}
				if(length(kk)==1){tmpS[TT,c_nC]<-tmpB[trunc(tt[1,j])+kk-1,c_CN]}
				if(length(kk)==2){tmpS[TT,c_nC]<-tmpS[TT,c_CN]}

			}else{
				if(max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk,c_CN]))<0.9){
					if(min(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk+1,c_CN]))>0.9){
						tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpB[kk,c_CN]
					}else{tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpS[TT:(TT+tt[2,j]-1),c_CN]}
				}else{
					if(min(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk,c_CN]))>0.9){
					if(max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk+1,c_CN]))<0.9){
						tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpB[kk+1,c_CN]
					}else{tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpS[TT:(TT+tt[2,j]-1),c_CN]}
					}else{tmpS[TT:(TT+tt[2,j]-1),c_nC]<-tmpS[TT:(TT+tt[2,j]-1),c_CN]}
				}
			}
			TT<-TT+tt[2,j]

#if one chromosome
			}else{
			ttchr<-numeric(0)
			for(p in 1:tt[2,j]){ttchr<-c(ttchr,trunc(tmpS[TT+p-1,c_chr]))}
			ttchr<-table(ttchr)
			p<-1
			for(p in 1:length(ttchr)){
				kkchr<-as.numeric(rownames(ttchr)[p])
				tt2j<-ttchr[p]
				kk1<-(-1)
				if(trunc(tmpB[kk,c_chr])==kkchr){kk1<-kk}
				if(trunc(tmpB[kk+1,c_chr])==kkchr){kk1<-kk+1}
				if(kk1==(-1)){

				}else{
					if(tt2j==1){
						if(abs(tmpS[TT,c_nC]-tmpB[kk1,c_CN])<0.9){tmpS[TT,c_nC]<-tmpB[kk1,c_CN]
						}else{tmpS[TT,c_nC]<-tmpS[TT,c_CN]}
					}else{
						if(max(abs(tmpS[TT:(TT+tt2j-1),c_nC]-tmpB[kk1,c_CN]))<0.9){
								tmpS[TT:(TT+tt2j-1),c_nC]<-tmpB[kk1,c_CN]
						}else{tmpS[TT:(TT+tt2j-1),c_nC]<-tmpS[TT:(TT+tt2j-1),c_CN]}
							
						}
					}

				TT<-TT+tt2j
			}#p

			}#in between chromosomes
			}# not a first segment
			}# in between segments
	}#j



tmpS<-getGTwithRank(tmpS,Centr_CL,NETM)

tmpS[,c_CN]<-tmpS[,c_CN1]
tmpS[,c_BA]<-tmpS[,c_BA1]

if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){dim_tmpS<-1}else{dim_tmpS<-dim(tmpS)[[1]]}
for(k in 1:dim_tmpS){
	kk<-which(tmp[,c_ind]==tmpS[k,c_ind])
	tmp[kk,]<-tmpS[k,]
}

}# if subclones


out<-tmp

}#function




###########################################################
####
####   Function shrinkReprTMP Uniting segments with equal c_conf column
####
###########################################################

shrinkReprTMP<-function(tmp){

tmp[1,c_lrr]<-tmp[1,c_len]*tmp[1,c_lrr]
for(k in 1:(dim(tmp)[[1]]-1)){
	if(tmp[k,c_conf]==tmp[k+1,c_conf]){
		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_lrr]<-tmp[k,c_lrr]+tmp[k+1,c_len]*tmp[k+1,c_lrr]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
		FL<-T
	}else{tmp[k,c_lrr]<-tmp[k,c_lrr]/tmp[k,c_len]
		tmp[k+1,c_lrr]<-tmp[k+1,c_len]*tmp[k+1,c_lrr]
		}

}#k
k<-dim(tmp)[[1]]
tmp[k,c_lrr]<-tmp[k,c_lrr]/tmp[k,c_len]

tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}

out<-tmp

}# function




################################
#######
#######  Function force a short (<50 SNPs)segments to long if possible
#######
########################################

fitShortSubclones<-function(tmp,Centr_CL,NETM,p_BAF,Length){


TMP<-tmp

tt<-which(TMP[,c_len]>=Length)
tt1<-table(TMP[,c_chr])

if(length(which(tt1==1))>0){tt1<-tt1[which(tt1==1)]
	tt1<-as.numeric(rownames(tt1))
	for(j in 1:length(tt1)){
		tt<-c(tt,which(TMP[,c_chr]==tt1[j]))
	}
tt<-as.numeric(rownames(table(tt)))
}

if(length(tt)>0){tMP<-TMP[-tt,];TMP<-TMP[tt,]}


TMP<-UniteSEGtmp_Illum(TMP,p_BAF)
dim(TMP)

TMP<-adjustTMPstableSubclone(TMP,Centr_CL,NETM,p_BAF)

TMP<-UniteSEGtmp_Illum(TMP,p_BAF)
dim(tMP)

tmp[,c_conf]<-0
for(k in 1:dim(TMP)[[1]]){
	tt<-which(tmp[,c_is]>=TMP[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=TMP[k,c_if])
	if(length(kk)>0){
		tmp[tt[kk],c_CN]<-TMP[k,c_CN]
		tmp[tt[kk],c_BA]<-TMP[k,c_BA]
		tmp[tt[kk],c_conf]<-k
	}
}
}

for(k in 1:dim(tMP)[[1]]){
	tt<-which(tmp[,c_ind]==tMP[k,c_ind])
	if(length(tt)>0){tmp[tt,c_conf]<-(-k)}
}



out<-tmp

}# function

###########################################################
####
####   Function shortBreaksToTMP small segments are fitted, if possible, to exact neihbours
####
###########################################################

shortBreaksToTMP<-function(tmp,CHR){
#CHR<-recCHR

##### Enumerating and attributing short segments to big and stable ones

ss<-data.matrix(table(tmp[,c_chr],sign(tmp[,c_conf])))
tt<-as.numeric(which(ss[,2]==0))
if(length(tt)>0){
	ss<-as.numeric(rownames(ss))[tt]
	k<-1
	for(k in 1:length(ss)){
		kk<-which(tmp[,c_chr]==ss[k])
		dd<-max(tmp[1:kk[1],c_conf])
		for(p in 1:length(kk)){tmp[kk[p],c_conf]<-dd+p}
		if(kk[length(kk)]<dim(tmp)[[1]]){
		tmp[(kk[length(kk)]+1):dim(tmp)[[1]],c_conf]<-tmp[(kk[length(kk)]+1):dim(tmp)[[1]],c_conf]+length(kk)
		}
	}#k
}

tt<-matrix(0,2,2)
t_pos<-0
t_chr<-1
k_neg<-0

k<-64
for(k in 1:dim(tmp)[[1]]){
  if(tmp[k,c_chr]==t_chr){
	if(tmp[k,c_conf]<0){k_neg<-k_neg+1
	}else{if(k_neg>0){
		if(is.na(t_pos)){tt<-cbind(tt,c(tmp[k,c_conf],k_neg));t_pos<-tmp[k,c_conf]
		}else{if(tmp[k,c_conf]==t_pos){tt<-cbind(tt,c(t_pos,k_neg))
			}else{tt<-cbind(tt,c((t_pos+tmp[k,c_conf])/2,k_neg));t_pos<-tmp[k,c_conf]}
			}
		k_neg<-0
		}else{t_pos<-tmp[k,c_conf]}
		}
  }else{if(k_neg>0){tt<-cbind(tt,c(t_pos,k_neg));k_neg<-0}
		t_pos<-NA;t_chr<-tmp[k,c_chr]
		if(tmp[k,c_conf]<0){k_neg<-k_neg+1}else{t_pos<-tmp[k,c_conf]}
	}
}#k

Rest0<-0

if(dim(tt)[[2]]>4){tt<-tt[,-c(1,2)];Rest0<-2}else{tt<-cbind(tt[,-c(1,2)],tt[,-c(1,2)]);Rest0<-1}

if(Rest0>0){

tt<-data.matrix(tt)
colnames(tt)<-seq(1,dim(tt)[[2]])

kk<-which(tmp[,c_conf]>0)
if(length(kk)>0){tmpS<-tmp[-kk,]
if((dim(tmp)[[1]]-length(kk)==1)||(Rest0==1)){tmpS<-rbind(tmpS,tmpS)}
tmpB<-tmp[kk,]
}

tmpB[1,c_is]<-1

for(k in 2:dim(tmpB)[[1]]){
	if(tmpB[k-1,c_chr]!=tmpB[k,c_chr]){
		kk<-which(as.numeric(names(CHR))==tmpB[k-1,c_chr])
		tmpB[k-1,c_if]<-CHR[kk]
		tmpB[k,c_is]<-CHR[kk]+1
	}
}

tmpB<-shrinkReprTMP(tmpB)
tt1<-which(tt[1,]==0.5)
if(length(tt1)>0){tt[1,tt1]<-1}

tmpS[,c_nS]<-1
tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))

TT<-1
for(j in 1:tt_end){
	
  if(tt[2,j]>1){tt1<-TT:(TT+tt[2,j]-1)}else{tt1<-TT}

  tmpS[tt1,c_bafH]<-tt[1,j]

	if(trunc(tt[1,j])==tt[1,j]){
		tmpS[tt1,c_CN]<-tmpB[tt[1,j],c_CN]
		tmpS[tt1,c_BA]<-tmpB[tt[1,j],c_BA]
	}
	TT<-TT+abs(tt[2,j])
}#j


####### Fit  short insertions
####### 1. Fit single supershort insertions within segment

tmpS[,c_nS]<-1
tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))
if((Rest0==1)&&(tt_end==2)){tt_end<-1}


TT<-1
j<-26
for(j in 1:tt_end){
	
  if(trunc(tt[1,j])==tt[1,j]){
   if(tt[2,j]==1){
	if(tmpS[TT,c_len]<10){tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])
	}else{
	  if(abs(tmpS[TT,c_nC]-tmpS[TT,c_CN])<1){
		if(tmpS[TT,c_len]<20){tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])
		}else{
			if(((tmpS[TT,c_CN]==tmpS[TT,c_CN1])&&((tmpS[TT,c_BA1]==0)||(tmpS[TT,c_BA1]==tmpS[TT,c_BA])||(tmpS[TT,c_BA]==0)))
				||((tmpS[TT,c_CN]==tmpS[TT,c_CN1+3])&&((tmpS[TT,c_BA1+3]==0)||(tmpS[TT,c_BA1+3]==tmpS[TT,c_BA])||(tmpS[TT,c_BA1+3]==0)))){
				tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])
			}else{}
			}
	  }else{if(min(tmpS[TT,c_nC],tmpS[TT,c_CN])>5){tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])}else{}

		}
		}
    }
  }
	TT<-TT+abs(tt[2,j])
}#j



Rest1<-0
kk<-which(tt[2,]<0)
if(length(kk)>0){
	if(dim(tt)[[2]]>length(kk)+1){tt<-tt[,-kk];Rest1<-2
	}else{if(dim(tt)[[2]]==length(kk)+1){
			if(Rest0==2){tt<-cbind(tt[,-kk],tt[,-kk]);Rest1<-1}
	}}
}

kk<-which(tmpS[,c_nS]==0)
if(length(kk)>0){
	if(dim(tmpS)[[1]]>length(kk)+1){tmpS<-tmpS[-kk,]
	}else{if(dim(tmpS)[[1]]==length(kk)+1){tmpS<-rbind(tmpS[-kk,],tmpS[-kk,])}
		}
}


####### 2. Fit single supershort insertions between segment

if(Rest1>0){

tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))
if((Rest1==1)&&(tt_end==2)){tt_end<-1}


TT<-1
j<-26

for(j in 1:tt_end){
	
  if(trunc(tt[1,j])!=tt[1,j]){
	kk<-trunc(tt[1,j])
	if(tt[2,j]==1){
	  if(tmpS[TT,c_len]<10){
		if(!is.na(tmpS[TT,c_lrr])){
			if(kk<dim(tmpB)[[1]]){
				pp<-which.min(c(abs(tmpS[TT,c_lrr]-tmpB[kk,c_lrr]),abs(tmpS[TT,c_lrr]-tmpB[kk+1,c_lrr])))
			}else{pp<-1}
			tmpS[TT,c_bafH]<-(-(kk+pp-1));tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])

		}else{tmpS[TT,c_bafH]<-(-kk);tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])}

	}else{
	  if(kk<dim(tmpB)[[1]]){

		if((abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])<1)&&(abs(tmpS[TT,c_nC]-tmpB[kk+1,c_CN])>=1))
				{tmpS[TT,c_bafH]<-(-kk);tmpS[TT,c_nS]<-(-1)}

		if((abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])>=1)&&(abs(tmpS[TT,c_nC]-tmpB[kk+1,c_CN])<1))
				{tmpS[TT,c_bafH]<-(-kk-1);tmpS[TT,c_nS]<-(-1)}

		if((abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])<1)&&(abs(tmpS[TT,c_nC]-tmpB[kk+1,c_CN])<1))
				{pp<-which.min(c(tmpS[TT,c_lrr],tmpS[TT,c_lrr])-tmpB[kk:(kk+1),c_lrr])
				tmpS[TT,c_bafH]<-(-(kk+pp-1));tmpS[TT,c_nS]<-(-1)}

		if((tmpS[TT,c_bafH]<0)&&(abs(tmpS[TT,c_bafH])==trunc(abs(tmpS[TT,c_bafH])))&&(tmpS[TT,c_len]<20)){
			tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])
			}


	  }else{if(abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])<1){tmpS[TT,c_bafH]<-(-kk);tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])}}
	}
	kk<-abs(tmpS[TT,c_bafH])
	if((trunc(kk)==kk)&&(kk!=0)){tmpS[TT,c_CN]<-tmpB[kk,c_CN];tmpS[TT,c_BA]<-tmpB[kk,c_BA]}
  }
}

	TT<-TT+abs(tt[2,j])
}#j


k<-66
if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){dim_tmpS<-1}else{dim_tmpS<-dim(tmpS)[[1]]}

for(k in 1:dim_tmpS){
	if((tmpS[k,c_bafH]<0)&&(tmpS[k,c_nS]==0)){
		tmpS[k,c_bafH]<-abs(tmpS[k,c_bafH])
		if(tmpS[k,c_is]==tmpB[tmpS[k,c_bafH],c_if]+1){
			tmpB[tmpS[k,c_bafH],c_if]<-tmpS[k,c_if]
			tmpB[tmpS[k,c_bafH],c_len]<-tmpB[tmpS[k,c_bafH],c_len]+tmpS[k,c_len]
		}else{
			if(tmpS[k,c_if]==tmpB[tmpS[k,c_bafH],c_is]-1){
				tmpB[tmpS[k,c_bafH],c_is]<-tmpS[k,c_is]
				tmpB[tmpS[k,c_bafH],c_len]<-tmpB[tmpS[k,c_bafH],c_len]+tmpS[k,c_len]
			}else{tmpS[k,c_nS]<-1;print("problem")}
			}
	}
}#k

Rest2<-0
kk<-which(tt[2,]<0)
if(length(kk)>0){
	if(dim(tt)[[2]]>length(kk)+1){tt<-tt[,-kk];Rest2<-2
	}else{if(dim(tt)[[2]]==length(kk)+1){
			if(Rest1==2){tt<-cbind(tt[,-kk],tt[,-kk]);Rest2<-1}
	}}
}else{Rest2<-Rest1}

kk<-which(tmpS[,c_nS]==0)
if(length(kk)>0){
	if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){Rest2<-0
	}else{if(dim(tmpS)[[1]]>length(kk)+1){tmpS<-tmpS[-kk,]
		}else{if(dim(tmpS)[[1]]==length(kk)+1){tmpS<-rbind(tmpS[-kk,],tmpS[-kk,])}
			}
		}
}


####### 3. Fit multiple insertions 
if(Rest2>0){

TT<-1
tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))
if((Rest2==1)&&(tt_end==2)){tt_end<-1}

j<-30

for(j in 1:tt_end){
	
  if(trunc(tt[1,j])==tt[1,j]){
	if(tt[2,j]>1){
		if((max(tmpS[TT:(TT+tt[2,j]-1),c_len])<10)&&(max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpS[TT:(TT+tt[2,j]-1),c_CN]),na.rm=T)<1)){
			tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0

		}else{tt1<-which(tmpS[TT:(TT+tt[2,j]-1),c_len]>=10)
 			if(length(tt1)>0){tt1<-TT+tt1-1

			if((max(abs(tmpS[tt1,c_nC]-tmpS[tt1,c_CN]))<1)&&(max(tmpS[tt1,c_len])<20)){
				tmpS[tt1,c_nS]<-0

			}else{p<-1
				for(p in 1:length(tt1)){
					if(tmpS[tt1[p],c_len]>20){
						if(((tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1])&&((tmpS[tt1[p],c_BA1]==0)||(tmpS[tt1[p],c_BA1]==tmpS[tt1[p],c_BA])||(tmpS[tt1[p],c_BA]==0)))
					||((tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1+3])&&((tmpS[tt1[p],c_BA1+3]==0)||(tmpS[tt1[p],c_BA1+3]==tmpS[tt1[p],c_BA])||(tmpS[tt1[p],c_BA1+3]==0)))){
							tmpS[tt1[p],c_nS]<-0
						}
					}else{if((tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1])||(tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1+3])){
							tmpS[tt1[p],c_nS]<-0}
						}
				}#p
				}
			}else{tt1<-which(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpS[TT:(TT+tt[2,j]-1),c_CN])>=1)
				if(length(tt1)<=1){tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0
}
				}
		}
	}
  }else{kk<-trunc(tt[1,j])
	if((tt[2,j]>1)&&(kk<dim(tmpB)[[1]])){

	   tt1<-which(tmpS[TT:(TT+tt[2,j]-1),c_len]>=10)

 	   if(length(tt1)==0){
		if((max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk,c_CN]),na.rm=T)<1)
			&&(min(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk+1,c_CN]),na.rm=T)>=1)){
			tmpS[TT:(TT+tt[2,j]-1),c_bafH]<-(-kk);tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0
		}

		if((max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk+1,c_CN]),na.rm=T)<1)
			&&(min(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk,c_CN]),na.rm=T)>=1)){
			tmpS[TT:(TT+tt[2,j]-1),c_bafH]<-(-kk-1);tmpB[TT:(TT+tt[2,j]-1),c_nS]<-0
		}

	   }else{   tt1<-TT+tt1-1	
			if((max(abs(tmpS[tt1,c_nC]-tmpB[kk,c_CN]))<1)&&(min(abs(tmpS[tt1,c_nC]-tmpB[kk+1,c_CN]))>=1))
				{tmpS[tt1,c_bafH]<-(-kk);tmpS[tt1,c_nS]<-0}

			if((min(abs(tmpS[tt1,c_nC]-tmpB[kk,c_CN]))>=1)&&(max(abs(tmpS[tt1,c_nC]-tmpB[kk+1,c_CN]))<1))
				{tmpS[tt1,c_bafH]<-(-kk-1);tmpS[tt1,c_nS]<-0}

			if(sum(tmpS[tt1,c_nS])!=0){
				t_adH<-tmpS[tt1[1],c_bafH]
					
				p<-0
				while(((p+1)<=length(tt1))&&(abs(tmpS[tt1[p+1],c_nC]-tmpB[kk,c_CN])<1)){
					tmpS[tt1[p+1],c_bafH]<-(-kk);tmpS[tt1[p+1],c_nS]<-(-1);p<-p+1}
				if(p!=0){p_up<-seq(1,p)}else{p_up<-numeric(0)}

				p<-0
				while(((p+1)<=length(tt1))&&(abs(tmpS[tt1[length(tt1)-p],c_nC]-tmpB[kk+1,c_CN])<1)){
					tmpS[tt1[length(tt1)-p],c_bafH]<-(-kk-1);tmpS[tt1[length(tt1)-p],c_nS]<-(-1);p<-p+1}
		
				if(p!=0){p_down<-seq(length(tt1)-p+1,length(tt1))}else{p_down<-numeric(0)}
					
				p_unkn<-intersect(p_up,p_down)
					
				if(length(p_unkn)>0){tmpS[tt1[p_unkn],c_bafH]<-(t_adH);tmpS[tt1[p_unkn],c_nS]<-1
				}else{p_unkn<-setdiff(seq(1,length(tt1)),union(p_up,p_down))
					if(length(p_unkn)>0){tmpS[tt1[p_unkn],c_bafH]<-(t_adH);tmpS[tt1[p_unkn],c_nS]<-1}
					}
			}	
		}

	}else{ if(tt[2,j]>1){tt1<-which(tmpS[TT:(TT+tt[2,j]-1),c_len]>=10)
			if(length(tt1)==0){
				if(max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpS[kk,c_CN]),na.rm=T)<1){
					tmpS[TT:(TT+tt[2,j]-1),c_bafH]<-(-kk);tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0
				}
			}else{if(max(abs(tmpS[tt1,c_nC]-tmpB[kk,c_CN]))<1){tmpS[tt1,c_bafH]<-(-kk);tmpS[tt1,c_nS]<-0}
				  }
		}
		}
	
#	for(p in TT:(TT+tt[2,j]-1)){
#	kk<-abs(tmpS[p,c_bafH])
#	if((trunc(kk)==kk)&&(kk!=0)){tmpS[p,c_CN]<-tmpB[kk,c_CN];tmpS[p,c_BA]<-tmpB[kk,c_BA]
#	}
#	}

	
  }#between segments
				
TT<-TT+abs(tt[2,j])
		
}#j


if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){dim_tmpS<-1}else{dim_tmpS<-dim(tmpS)[[1]]}

k<-154
for(k in 1:dim_tmpS){
	if((tmpS[k,c_bafH]<0)&&(tmpS[k,c_nS]==0)){
		tmpS[k,c_bafH]<-abs(tmpS[k,c_bafH])
		if(tmpS[k,c_is]>=tmpB[tmpS[k,c_bafH],c_if]+1){
		tmpB[tmpS[k,c_bafH],c_if]<-tmpS[k,c_if]
		tmpB[tmpS[k,c_bafH],c_len]<-tmpB[tmpS[k,c_bafH],c_len]+tmpS[k,c_len]
		}else{
		if(tmpS[k,c_if]<=tmpB[tmpS[k,c_bafH],c_is]-1){
		tmpB[tmpS[k,c_bafH],c_is]<-tmpS[k,c_is]
		tmpB[tmpS[k,c_bafH],c_len]<-tmpB[tmpS[k,c_bafH],c_len]+tmpS[k,c_len]
		}else{
		if((tmpS[k,c_is]>=tmpB[tmpS[k,c_bafH],c_is])&&(tmpS[k,c_if]<=tmpB[tmpS[k,c_bafH],c_if])){
			tmpB[tmpS[k,c_bafH],c_len]<-tmpB[tmpS[k,c_bafH],c_len]+tmpS[k,c_len]
		}else{print("problem")}
		}
		}
	}
}#k


Rest3<-2

kk<-which(tmpS[,c_nS]==0)

if(length(kk)>0){
	if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){Rest3<-0
	}else{if(dim(tmpS)[[1]]>length(kk)+1){tmpS<-tmpS[-kk,]
		}else{if(dim(tmpS)[[1]]==length(kk)+1){tmpS<-rbind(tmpS[-kk,],tmpS[-kk,]);Rest3<-1}
			}
		}
}else{if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){Rest3<-1}}



#### Adjust tmpS with new copy numbers

if(Rest3>0){

tt<-which(tmpS[,c_nS]<0)

if(length(tt)>0){

	tmpS[tt,c_nC]<-tmpB[abs(tmpS[tt,c_bafH]),c_CN]

	if(length(tt)>1){tmpO<-tmpS[tt,]}else{tmpO<-rbind(tmpS[tt,],tmpS[tt,])}

	tmpO<-getGTwithRank(tmpO,Centr_CL,NETM)

	if(length(tt)>1){
		tmpS[tt,c_CN1:(c_CN1+8)]<-tmpO[,c_CN1:(c_CN1+8)]
		tmpS[tt,c_CN]<-tmpO[,c_CN1]
		tmpS[tt,c_BA]<-tmpO[,c_BA1]
		}else{
		tmpS[tt,c_CN1:(c_CN1+8)]<-tmpO[1,c_CN1:(c_CN1+8)]
		tmpS[tt,c_CN]<-tmpO[1,c_CN1]
		tmpS[tt,c_BA]<-tmpO[1,c_BA1]
		}

	for(p in 1:length(tt)){
		if(((tmpS[tt[p],c_CN]==tmpS[tt[p],c_CN1])&&((tmpS[tt[p],c_BA1]==0)||(tmpS[tt[p],c_BA1]==tmpS[tt[p],c_BA])||(tmpS[tt[p],c_BA]==0)))
		||((tmpS[tt[p],c_CN]==tmpS[tt[p],c_CN1+3])&&((tmpS[tt[p],c_BA1+3]==0)||(tmpS[tt[p],c_BA1+3]==tmpS[tt[p],c_BA])||(tmpS[tt[p],c_BA1+3]==0)))){
							tmpS[tt[p],c_nS]<-0
		}
	}


for(k in 1:dim(tmpS)[[1]]){
	if((tmpS[k,c_bafH]<0)&&(tmpS[k,c_nS]==0)){
		tmpS[k,c_bafH]<-abs(tmpS[k,c_bafH])
		if(tmpS[k,c_is]>=tmpB[tmpS[k,c_bafH],c_if]+1){
		tmpB[tmpS[k,c_bafH],c_if]<-tmpS[k,c_if]
		tmpB[tmpS[k,c_bafH],c_len]<-tmpB[tmpS[k,c_bafH],c_len]+tmpS[k,c_len]
		}else{
		if(tmpS[k,c_if]<=tmpB[tmpS[k,c_bafH],c_is]-1){
		tmpB[tmpS[k,c_bafH],c_is]<-tmpS[k,c_is]
		tmpB[tmpS[k,c_bafH],c_len]<-tmpB[tmpS[k,c_bafH],c_len]+tmpS[k,c_len]
		}else{
		if((tmpS[k,c_is]>=tmpB[tmpS[k,c_bafH],c_is])&&(tmpS[k,c_if]<=tmpB[tmpS[k,c_bafH],c_if])){
			tmpB[tmpS[k,c_bafH],c_len]<-tmpB[tmpS[k,c_bafH],c_len]+tmpS[k,c_len]
		}else{print("problem")}

		}
		}
	}
}#k


kk<-which(tmpS[,c_nS]==0)
if(length(kk)>0){tmpS<-tmpS[-kk,]}

}# if tt>0

}# Rest3
}# Rest2
}# Rest1

#############################
### resulting tmp 

tmp[,c_conf]<-0
for(k in 1:dim(tmpB)[[1]]){
	tt<-which(tmp[,c_is]>=tmpB[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=tmpB[k,c_if])
	if(length(kk)>0){
		tmp[tt[kk],c_CN]<-tmpB[k,c_CN]
		tmp[tt[kk],c_BA]<-tmpB[k,c_BA]
		tmp[tt[kk],c_conf]<-k
	}
}
}


if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){dim_tmpS<-1}else{dim_tmpS<-dim(tmpS)[[1]]}
for(k in 1:dim_tmpS){
	tt<-which(tmp[,c_ind]==tmpS[k,c_ind])
	if(length(tt)==1){tmp[tt,c_conf]<-(-k)
		tmp[tt,c_CN]<-tmpS[k,c_CN1]
		tmp[tt,c_BA]<-tmpS[k,c_BA1]
	
	}
}

}# Rest0

out<-tmp

}#function



###########################################################
####
####   Function iliminateShort_FinalPolish Uniting segments <10 SNPs 
####
###########################################################

iliminateShort_FinalPolish<-function(tmp,Short){

for(k in 2:(dim(tmp)[[1]]-1)){

	if((tmp[k,c_conf]!=0)&&(tmp[k,c_len]<Short)&&(tmp[k-1,c_chr]==tmp[k+1,c_chr])){
	if((tmp[k,c_conf]==tmp[k-1,c_conf])&&(tmp[k,c_conf]==tmp[k+1,c_conf])){
	if((is.na(tmp[k,c_lrr]))||(abs(tmp[k,c_lrr]-tmp[k+1,c_lrr])>abs(tmp[k,c_lrr]-tmp[k-1,c_lrr]))){
		tmp[k-1,c_if]<-tmp[k,c_if]
		tmp[k-1,c_len]<-tmp[k-1,c_len]+tmp[k,c_len]
		tmp[k,]<-tmp[k-1,]
		tmp[k-1,1]<-0
	}else{if(abs(tmp[k,c_lrr]-tmp[k+1,c_lrr])<=abs(tmp[k,c_lrr]-tmp[k-1,c_lrr])){
		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
		}}
}}
}#k
tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}

for(k in 2:dim(tmp)[[1]]){
	if((tmp[k,c_conf]!=0)&&(tmp[k,c_len]<Short)&&(tmp[k,c_chr]==tmp[k-1,c_chr])&&(tmp[k,c_conf]==tmp[k-1,c_conf])){
		tmp[k-1,c_if]<-tmp[k,c_if]
		tmp[k-1,c_len]<-tmp[k-1,c_len]+tmp[k,c_len]
		tmp[k,]<-tmp[k-1,]
		tmp[k-1,1]<-0
	}
}#k
tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}

for(k in 1:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_conf]!=0)&&(tmp[k,c_len]<Short)&&(tmp[k,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_conf]==tmp[k+1,c_conf])){
		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
	}
}#k
tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}

k<-412
for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_conf]!=0)&&(tmp[k,c_len]<Short)&&(tmp[k-1,c_chr]==tmp[k+1,c_chr])){

	if(is.na(tmp[k+1,c_lrr])){
		tmp[k,c_if]<-tmp[k+1,c_if]
		tmp[k,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k+1,]<-tmp[k,]
		tmp[k,]<-tmp[k-1,]
		tmp[k-1,1]<-0
	}else{

	if((is.na(tmp[k,c_lrr]))||(abs(tmp[k,c_lrr]-tmp[k+1,c_lrr])>abs(tmp[k,c_lrr]-tmp[k-1,c_lrr]))){
		tmp[k-1,c_if]<-tmp[k,c_if]
		tmp[k-1,c_len]<-tmp[k-1,c_len]+tmp[k,c_len]
		tmp[k,]<-tmp[k-1,]
		tmp[k-1,1]<-0

	}else{if(abs(tmp[k,c_lrr]-tmp[k+1,c_lrr])<=abs(tmp[k,c_lrr]-tmp[k-1,c_lrr])){
		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
		}}
	}
	}
}#k
tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}

tmp[,1]<-seq(1,dim(tmp)[[1]])
tt<-which(tmp[,c_conf]<=0)
if(length(tt)>0){tmp[tt,c_conf]<-(-seq(1,length(tt)))}

out<-tmp

}#function


###########################################################
####
####   Function updateMeans_FinalPolish
####
###########################################################

updateMeans_FinalPolish<-function(tmp,datA,datL,germHomoThr){

tt<-which(is.na(datA));if(length(tt)!=0){datA[tt]<-1}
tt<-which((datA>1));if(length(tt)!=0){datA[tt]<-1}

k<-1
for(k in 1:dim(tmp)[[1]]){
	if(tmp[k,c_if]>tmp[k,c_is]+10){
	tmp[k,c_lrr]<-median(datL[tmp[k,c_is]:tmp[k,c_if]],na.rm=T)
	}else{if(tmp[k,c_if]>tmp[k,c_is]){
		tmp[k,c_lrr]<-mean(datL[tmp[k,c_is]:tmp[k,c_if]],na.rm=T)
		}else{tmp[k,c_lrr]<-datL[tmp[k,c_is]]}	
		}
}#k

ad<-datA
tt<-which(datA>germHomoThr);if(length(tt)!=0){ad[tt]<-NA}

k<-14
for(k in 1:dim(tmp)[[1]]){

  adv<-ad[tmp[k,c_is]:tmp[k,c_if]]
  tt<-which(!is.na(adv))

  if(length(tt)>length(adv)/8){

	adv<-adv[tt]
	if(length(adv)>30){

	  tmp[k,c_baf]<-median(adv,na.rm=T)
	  if(tmp[k,c_baf]<0.6){
		hh<-hist(adv, breaks=seq(0.5,1,0.02),right=F,plot=F)
		
		if((sum(hh$counts[1:3])/sum(hh$counts)>0.7)&&((hh$counts[1]+hh$counts[2])>2*hh$counts[3]*1.2)){
			if(hh$counts[1]>hh$counts[2]){tmp[k,c_baf]<-0.5}else{tmp[k,c_baf]<-0.52}
		}else{dd<-density(adv,n=20)
			tmp[k,c_baf]<-dd$x[which.max(dd$y)]
			}
	  }
	
	}else{if(length(adv)>0){hh<-hist(adv, breaks=seq(0.5,1,0.05),plot=F)
			tmp[k,c_baf]<-hh$mids[which.max(hh$counts)]
		}else{tmp[k,c_baf]<-NA}
		}

  }else{tt<-which(is.na(adv))
	adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
	if(length(tt)>10){
		adv<-adv[tt]
		if(length(adv)>30){
			dd<-density(adv,n=20)
			tmp[k,c_baf]<-dd$x[which.max(dd$y)]
		}else{hh<-hist(adv, breaks=seq(0.5,1,0.05),right=F,plot=F)
			tmp[k,c_baf]<-hh$mids[which.max(hh$counts)]}
	}else{
		if(length(tt)>0){
			adv<-adv[tt]
			hh<-hist(adv, breaks=seq(0.5,1,0.1),plot=F)
			tmp[k,c_baf]<-hh$mids[which.max(hh$counts)]
		}else{tmp[k,c_baf]<-NA}
		}
	}
 
}#k


tmp<-round(tmp,4)

}#function





#################################################
############## Function assigns the number of region
#############
#################################################

getConfidence<-function(tmp){

tmp[,c_conf]<-0
tmp[1,c_conf]<-1

for(k in 2:dim(tmp)[[1]]){
	if((tmp[k,c_chr]==tmp[k-1,c_chr])&&(tmp[k,c_CN]==tmp[k-1,c_CN])&&(tmp[k,c_BA]==tmp[k-1,c_BA])){
		tmp[k,c_conf]<-tmp[k-1,c_conf]
	}else{tmp[k,c_conf]<-tmp[k-1,c_conf]+1}
}#k

out<-tmp

}# function



########################################################################
############
############  Output PRELIMINARY plotGAP
############
########################################################################

GAPplot_preliminary<-function(tmp,outputPath_GAP,nameSample,Centr_CL,p_BAF,q_LRR){

jpeg(file=paste(outputPath_GAP,paste("GAP_plot_",nameSample,".jpg",sep=""),sep="/"),height=1200,width=1000,pointsize=24)

plot(tmp[,c_baf],tmp[,c_lrr],main=paste("GAP for ", nameSample),xlim=c(0,1),ylim=c(-1,1),type="n",xlab="Allelic difference",ylab="Log ratio")

CEX<-tmp[,c_len]/quantile(tmp[,c_len],prob=0.7)
tt<-which(CEX>4)
if(length(tt)>0){CEX[tt]<-4}

tt<-which(CEX<0.03)
if(length(tt)>0){CEX[tt]<-0}

points(1-tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="black")
points(tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="grey")

abline(h=0)
abline(v=0.5)

############################
######### GAP pattern

RB<-rainbow(40)

for(p in 1:25){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){
		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
	}
}

for(p in 31:38){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){
		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
	}
}

for(j in 1:dim(Centr_CL)[[2]]){

	if(Centr_CL[1,j]<8){
	if(Centr_CL[1,j]==Centr_CL[2,j]*2){
		text(0.54,Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
	}else{
		text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
		}
}
}

text(0.1,-0.8,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.99)
text(0.1,-0.9,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.99)
text(0.1,-1,paste("2copyLRR = ",as.character(Centr_CL[5,4])),cex=0.99)

dev.off()


}## GAP plot function





########################################################################
############
############  Output summary plotGAP
############
########################################################################

GAPplot<-function(tmp,nameSample,SNP_annot, Centr_CL){

##################### Centromere summary

tt<-sort(union(which(tmp[,c_len]>15),which(tmp[,c_chr]==21)))
tt<-sort(union(tt,which(trunc(tmp[,c_chr])==24)))

if(length(tt)>0){tmp1<-tmp[tt,]}


Chr_counts<-0
Centr_counts<-0
centr<-tmp1[1,]

for(k in 1:(dim(tmp1)[[1]]-1)){
	if((tmp1[k+1,c_chr]>tmp1[k,c_chr])&&(round((tmp1[k+1,c_chr]-0.2),0)==tmp1[k,c_chr])){
		centr<-rbind(centr,tmp1[k:(k+1),])
		Chr_counts<-Chr_counts+(tmp1[k,c_CN]+tmp1[k+1,c_CN])/2
		Centr_counts<-Centr_counts+tmp1[k,c_CN]+tmp1[k+1,c_CN]
	}else{
		tt<-round((tmp1[k+1,c_chr]-0.2),0)
		if(tmp1[k+1,c_chr]>tmp1[k,c_chr]){
			if((tt==13)||(tt==14)||(tt==15)||(tt==21)||(tt==22)){
				centr<-rbind(centr,tmp1[k+1,])
				Chr_counts<-Chr_counts+tmp1[k+1,c_CN]
				Centr_counts<-Centr_counts+tmp1[k+1,c_CN]

			}
		}
	}
}#k

centr<-centr[-1,]
centr<-cbind(centr[,1:7],centr[,16:17])

####################### DNA index

C_Ind<-1;C_Chr<-2;C_Pos<-4

DNAi<-0
tt<-0
k<-1
for(k in 1:dim(tmp)[[1]]){
DNAi<-DNAi+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000*tmp[k,c_CN]
tt<-tt+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
}
DNAi<-round(DNAi/tt/2,2)

#######################################################################
###########
########### Summary plot
###########
#######################################################################


jpeg(file=paste(outputPath,paste("GAP_plot_",nameSample,".jpg",sep=""),sep="/"),height=1200,width=1000,pointsize=24)

plot(tmp[,c_baf],tmp[,c_lrr],main=paste("GAP for ", nameSample),xlim=c(0,1),ylim=c(-1,1),type="n",xlab="Allelic difference",ylab="Log ratio")

CEX<-tmp[,c_len]/quantile(tmp[,c_len],prob=0.7)
tt<-which(CEX>4)
if(length(tt)>0){CEX[tt]<-4}

tt<-which(CEX<0.03)
if(length(tt)>0){CEX[tt]<-0}

points(1-tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="black")
points(tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="grey")
#points(tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="black")

abline(h=0)
abline(v=0.5)

############################
######### GAP pattern

RB<-rep("black",40)
RB<-rainbow(40)


for(p in 1:25){
#for(p in 2:12){

	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){
		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
#		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2])+0.05,max(NETM[tt1,1]),border=RB[p])

}
}

for(p in 31:38){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){
		rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p])
	}
}

for(j in 1:dim(Centr_CL)[[2]]){

	if(Centr_CL[1,j]<8){
	if(Centr_CL[1,j]==Centr_CL[2,j]*2){
		text(0.54,Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
	}else{
		text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""))
		}
}
}

tt<-which(tmp[,c_len]<30);if(length(tt)>0){tmp1<-tmp[-tt,]}
tmp1<-shrinkReprTMP(tmp1)


text(0.1,-0.8,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.99)
text(0.1,-0.9,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.99)
text(0.1,-1,paste("DNAind = ",as.character(DNAi)),cex=0.99)

text(0.35,-0.8,paste("Breaks = ",as.character(dim(tmp1)[[1]]-41)),cex=0.99)
text(0.35,-0.9,paste("Chrs = ",as.character(Chr_counts)),cex=0.99)
text(0.35,-1,paste("Centrs = ",as.character(Centr_counts)),cex=0.99)

dev.off()


}## GAP plot function



########################################################################
############
############  Output summary plots:Chromosome pictures
############
########################################################################

summaryPlots_perChrom<-function(tmp,nameSample,SNP_annot, datA, datL, Centr_CL){

C_Ind<-1;C_Chr<-2;C_Pos<-4

Plot_dir<-paste(outputPath,paste("Chr_Plots_",nameSample,sep=""),sep="/")
dir.create(Plot_dir, showWarnings =F, recursive = FALSE, mode = "0777")

k<-1
for(k in 1:23){

jpeg(file=paste(Plot_dir,"/Plot_",nameSample,"_Chr_",
   as.character(k),".jpg",sep=""),height=1000,width=1200,pointsize=24)

MAIN<-paste(nameSample,"   Chr_",as.character(k))

KN<-which(SNP_annot[,C_Chr]==k)
KK<-KN[length(KN)]
KN<-KN[1]

plot(c(1,SNP_annot[KK,C_Pos]/1000),c(0,0),ylim=c(-15,15),main=MAIN,type="n",xlab="kb",ylab="",yaxt="n")
points(SNP_annot[KN:KK,C_Pos]/1000,datL[KN:KK]*4-9,col="black",pch=".")

KN<-which(SNP_annot[,C_Chr]==k)
KK<-KN[length(KN)]
KN<-KN[1]

points(SNP_annot[KN:KK,C_Pos]/1000,6*(0.5+(datA[KN:KK]-0.5)*sample(c(-1,1),(KK-KN+1),replace=T))+7,col="black",pch=".")

abline(h=(-2),col="black")
abline(h=0,col="black")
text(-4000,0,"4")
text(-4000,-2,"2")


# Recognized profile tmp

KN<-which(trunc(tmp[,c_chr])==k)
KK<-KN[length(KN)]
KN<-KN[1]

p<-KN
for(p in KN:KK){
if(tmp[p,c_len]>20){
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_lrr]*4-9,tmp[p,c_lrr]*4-9),col="blue",lwd=3)
}
}

# Recognized profiles TMP

KN<-which(trunc(tmp[,c_chr])==k)
KK<-KN[length(KN)]
KN<-KN[1]

p<-KN
for(p in KN:KK){
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_lrr]*4-9,tmp[p,c_lrr]*4-9),col="red",type="l",lwd=3)
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_baf],tmp[p,c_baf])*6+7,col="red",lwd=3)
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(1-tmp[p,c_baf],1-tmp[p,c_baf])*6+7,col="red",lwd=3)
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_CN],tmp[p,c_CN])-4,col="blue",lwd=3)
}

# Connecting profile
if(KK>(KN+1)){
for(p in (KN+1):KK){
lines(c(SNP_annot[tmp[p-1,c_if],C_Pos]/1000,SNP_annot[tmp[p,c_is],C_Pos]/1000),c(tmp[p-1,c_CN],tmp[p,c_CN])-4,col="blue",lwd=1)
}
}

# Acquired homozygosity regions
for(p in KN:KK){
if(tmp[p,c_CN]==tmp[p,c_BA]){
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_CN],tmp[p,c_CN])-4,col="red",lwd=3)
}
}

# Germline homozygosity regions
for(p in KN:KK){
if(tmp[p,c_BA]==0){
lines(c(SNP_annot[tmp[p,c_is],C_Pos]/1000,SNP_annot[tmp[p,c_if],C_Pos]/1000),c(tmp[p,c_CN],tmp[p,c_CN])-4,col="green",lwd=3)
}
}


p<-82
for(p in KN:KK){

bb<-""

	if((tmp[p,c_BA]==0)||(tmp[p,c_BA]==tmp[p,c_CN])){
		if(tmp[p,c_CN]>1){
			for(s in 1:tmp[p,c_CN]){bb<-paste(bb,"A",sep="")}
		}else{if(tmp[p,c_CN]==1){bb<-"A"}else{bb<-"0"}}
	}else{
		if(tmp[p,c_BA]>1){
			for(s in 1:tmp[p,c_BA]){bb<-paste(bb,"A",sep="")}
		}else{if(tmp[p,c_BA]==1){bb<-"A"}}

		if(tmp[p,c_CN]>tmp[p,c_BA]+1){
			for(s in (tmp[p,c_BA]+1):tmp[p,c_CN]){bb<-paste(bb,"B",sep="")}
		}else{if(tmp[p,c_CN]==tmp[p,c_BA]+1){bb<-paste(bb,"B",sep="")}}
	}



if((tmp[p,c_if]-tmp[p,c_is])/2>200){

if(tmp[p,c_is]==1){text((SNP_annot[tmp[p,c_is],C_Pos]+SNP_annot[tmp[p,c_if],C_Pos])/2000,tmp[p,c_CN]-4.5,bb,cex=0.5)
}else{
text((SNP_annot[tmp[p,c_is],C_Pos]+SNP_annot[tmp[p,c_if],C_Pos])/2000,tmp[p,c_CN]-3.3,bb,cex=0.5)
}
}

}#p

dev.off()

}#k


}# function


########################################################################
############
############  Output summary plots
############
########################################################################

summaryPlots<-function(tmp,nameSample,SNP_annot, datA, datL, Centr_CL){


C_Ind<-1;C_Chr<-2;C_Pos<-4


############# Centromere summary
tt<-sort(union(which(tmp[,c_len]>15),which(tmp[,c_chr]==21)))
tt<-sort(union(tt,which(trunc(tmp[,c_chr])==24)))

if(length(tt)>0){tmp1<-tmp[tt,]}



Chr_counts<-0
Centr_counts<-0
centr<-tmp1[1,]

for(k in 1:(dim(tmp1)[[1]]-1)){
	if((tmp1[k+1,c_chr]>tmp1[k,c_chr])&&(round((tmp1[k+1,c_chr]-0.2),0)==tmp1[k,c_chr])){
		centr<-rbind(centr,tmp1[k:(k+1),])
		Chr_counts<-Chr_counts+(tmp1[k,c_CN]+tmp1[k+1,c_CN])/2
		Centr_counts<-Centr_counts+tmp1[k,c_CN]+tmp1[k+1,c_CN]
	}else{
		tt<-round((tmp1[k+1,c_chr]-0.2),0)
		if(tmp1[k+1,c_chr]>tmp1[k,c_chr]){
			if((tt==13)||(tt==14)||(tt==15)||(tt==22)){
				centr<-rbind(centr,tmp1[k+1,])
				Chr_counts<-Chr_counts+tmp1[k+1,c_CN]
				Centr_counts<-Centr_counts+tmp1[k+1,c_CN]

			}
		}
	}
}#k

centr<-centr[-1,]

#######


tt<-table(SNP_annot[,C_Chr],SNP_annot[,C_Chr+1])
CHR<-c(0)
tCHR<-c(0)

for(k in 1:dim(tt)[[1]]){
CHR<-c(CHR,CHR[2*k-1]+tt[k,1])
CHR<-c(CHR,CHR[2*k]+tt[k,2])
tCHR<-c(tCHR,as.numeric(rownames(tt))[k])
tCHR<-c(tCHR,as.numeric(rownames(tt))[k]+0.5)
}

names(CHR)<-tCHR
rm(tCHR)

k<-2
while(k<=(length(CHR)-1)){
if((CHR[k]-CHR[k-1]<3)||(CHR[k+1]-CHR[k]<3)){
CHR[k]<-CHR[k+1]
names(CHR)[k+1]<-0
}
k<-k+2
}

if(length(which(names(CHR)=="0"))){
CHR<-CHR[-which(names(CHR)=="0")]}

CHR<-c(0,CHR)
names(CHR)[1]<-"0"


#######################################################################
###########
########### Summary plot
###########
#######################################################################


jpeg(file=paste(outputPath,paste("Summary_plot_",nameSample,".jpg",sep=""),sep="/"),height=2000,width=1600,pointsize=24)

split.screen(c(2,1)) 
screen(1)

plot(c(-6000,max(tmp[,c_if])+2000),c(-3,3),type="n",main=nameSample,xlab="Probe Index along Genome",ylab="",lab=c(7,1,7))

####### Plotting genome

#RB<-rainbow(length(CHR), s = 1, v = 1, start = 0, end = max(1,length(CHR) - 1)/length(CHR),gamma = 1, alpha = 1)

for(k in 2:(length(CHR)-2)){

rect(CHR[k],1.3,CHR[k]-1,2.6, col="black",border="black")
rect(CHR[k],-2.2,CHR[k]-1,-0.8, col="black",border="black")

if(round((as.numeric(names(CHR))[k]-0.2),0)%%2==0){
	rect(CHR[k-1]+1,1.3,CHR[k],2.8, col="grey90",border="grey90")
	rect(CHR[k-1]+1,-2.2,CHR[k],-0.8, col="grey90",border="grey90")
	if(round((as.numeric(names(CHR))[k])-0.2,0)==as.numeric(names(CHR))[k-1]){
		rect(CHR[k-1],1.3,CHR[k-1]-1,2.8, col="black",border="black")
		rect(CHR[k-1],-2.2,CHR[k-1]-1,-0.8, col="black",border="black")
}}

if(round((as.numeric(names(CHR))[k]),0)==as.numeric(names(CHR))[k]){
if(as.numeric(names(CHR))[k]%%2==0){
	text((CHR[k-1]+CHR[k])/2,-2.5,names(CHR)[k],col="black",cex=0.7)
}else{text((CHR[k-1]+CHR[k])/2,3,names(CHR)[k],col="black",cex=0.7)}
}
#rect(CHR[k-1],-2.8,CHR[k],-3, col=RB[k],border=RB[k])
}

RB<-rainbow(40)

for(p in 1:dim(tmp)[[1]]){
	cl<-which(abs(Centr_CL[1,]-tmp[p,c_CN])==0)
	ba<-which(abs(Centr_CL[2,cl]-tmp[p,c_BA])==0)
	if(length(ba)>0){tmp[p,c_sb]<-Centr_CL[3,cl[ba[1]]]}else{tmp[p,c_sb]<-30+tmp[p,c_CN]}
}


tt<-which(SNP_annot[,C_Ind]%%10==0)
points(SNP_annot[tt,C_Ind],datL[tt]/2-1.5,col="black",pch=".")

tt<-which(datA<0.97)
tt<-tt[which(tt%%2==0)]
points(SNP_annot[tt,C_Ind],(0.5+(datA[tt]-0.5)*sample(c(-1,1),(length(datA[tt])),replace=T))+1.5,col="black",pch=".")
lines(c(tmp[1,c_is],tmp[dim(tmp)[[1]],c_if]),c(1.5,1.5),col="black",type="l",lwd=2)
lines(c(tmp[1,c_is],tmp[dim(tmp)[[1]],c_if]),c(2.5,2.5),col="black",type="l",lwd=2)


for(p in 1:dim(tmp)[[1]]){
rect(tmp[p,c_is],-2.8,tmp[p,c_if],-3, col=RB[tmp[p,c_sb]],border=RB[tmp[p,c_sb]])
if(tmp[p,c_len]>50){
lines(c(tmp[p,c_is],tmp[p,c_if]),c(tmp[p,c_lrr],tmp[p,c_lrr])/2-1.5,col="red",type="l",lwd=2)
lines(c(tmp[p,c_is],tmp[p,c_if]),c(tmp[p,c_baf],tmp[p,c_baf])+1.5,col="red",lwd=2)
lines(c(tmp[p,c_is],tmp[p,c_if]),1-c(tmp[p,c_baf],tmp[p,c_baf])+1.5,col="red",lwd=2)
}
}

abline(h=(-1),col="grey60",lty=2)
abline(h=(-1.5),col="white")
abline(h=(-2),col="grey60",lty=2)

abline(h=0,col="blue",lty=1)
abline(h=-0.2,col="blue",lty=2)
abline(h=-0.4,col="blue",lty=2)
abline(h=0.2,col="blue",lty=2)
abline(h=0.4,col="blue",lty=1)
abline(h=0.6,col="blue",lty=2)
abline(h=0.8,col="blue",lty=2)
abline(h=1,col="blue",lty=2)

rect(-30000,-4,-2000,4, col="white",border="white")
rect(max(tmp[,c_if])+2000,-4,max(tmp[,c_if])+30000,4, col="white",border="white")


text(-10000,1.5,"0",col="black")
text(-10000,2.5,"1",col="black")
text(-10000,3,"BAF",col="black")
text(-10000,1.1,"Copy",col="black")
text(-10000,-1.5,"0",col="black")
text(-10000,-1,"1",col="black")
text(-10000,-2,"-1",col="black")
text(-10000,-0.6,"LRR",col="black")
text(-10000,0,"2",col="black")
text(-10000,0.4,"4",col="black")


# Copy number
tt<-which(tmp[,c_len]<100);if(length(tt)>0){tmp1<-tmp[-tt,]}

for(p in 1:dim(tmp1)[[1]]){
lines(c(tmp1[p,c_is],tmp1[p,c_if]),c(tmp1[p,c_CN],tmp1[p,c_CN])/5-0.4,col="navy",type="l",lwd=3)
}

for(p in 2:dim(tmp1)[[1]]){
lines(c(tmp1[p-1,c_if],tmp1[p,c_is]),c(tmp1[p-1,c_CN],tmp1[p,c_CN])/5-0.4,col="navy",lwd=1)
}


# Acquired homozygosity regions
for(p in 2:dim(tmp1)[[1]]){

if((tmp1[p,c_CN]==tmp1[p,c_BA])||(tmp1[p,c_BA]==0)){
lines(c(tmp1[p,c_is],tmp1[p,c_if]),c(tmp1[p,c_CN],tmp1[p,c_CN])/5-0.4,col="red",lwd=3)
}
}


######## GAP figure

screen(2)
split.screen(c(1,2))
screen(3)


plot(tmp[,c_baf],tmp[,c_lrr],main="GAP",xlim=c(0,1),ylim=c(-1,1),type="n",xlab="B Allele Frequency (BAF)",ylab="Log R ratio (LRR)")

CEX<-tmp[,c_len]/quantile(tmp[,c_len],prob=0.7)

tt<-which(CEX>5)
if(length(tt)>0){CEX[tt]<-5}

tt<-which(CEX<0.04)
if(length(tt)>0){CEX[tt]<-0}

points(1-tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="black")
points(tmp[,c_baf],tmp[,c_lrr],cex=CEX,col="grey")

abline(h=0)
abline(v=0.5)


################ 
tt<-which(tmp[,c_len]<50);if(length(tt)>0){tmp1<-tmp[-tt,]}

############################
######### GAP pattern

RB<-rainbow(40)

for(p in 1:25){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p],density=80)}
}

for(p in 31:38){
	tt1<-which(NETM[,3]==p)
	if(length(tt1)>0){rect(min(NETM[tt1,2]),min(NETM[tt1,1]),max(NETM[tt1,2]),max(NETM[tt1,1]), col=RB[p],border=RB[p],density=80)}
}

for(j in 1:dim(Centr_CL)[[2]]){
if(Centr_CL[1,j]<=7){
	if(Centr_CL[1,j]==Centr_CL[2,j]*2){	text(0.55,Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""),font=2)
	}else{text(Centr_CL[4,j],Centr_CL[5,j],paste(as.character(Centr_CL[1,j]),"/",as.character(Centr_CL[2,j]),sep=""),font=2)}
}}


text(0.3,-0.8,paste("p_BAF = ",as.character(round(p_BAF,2)),sep=""),cex=0.8)
text(0.3,-0.9,paste("q_LRR = ",as.character(round(q_LRR,2)),sep=""),cex=0.8)
text(0.3,-1,paste("2copyLRR = ",as.character(round(Centr_CL[5,4],3)),sep=""),cex=0.8)



################ Centromere table and summary inf
################## Summary inf

DNAi<-0
tt<-0
k<-1
for(k in 1:dim(tmp)[[1]]){
DNAi<-DNAi+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000*tmp[k,c_CN]
tt<-tt+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
}
DNAi<-round(DNAi/tt/2,2)

HomoP<-0
for(k in 1:dim(tmp)[[1]]){
  if((tmp[k,c_CN]==tmp[k,c_BA])||(tmp[k,c_BA]==0)){
	HomoP<-HomoP+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
  }
}
HomoP<-round(HomoP/tt*100,0)

BaseP<-0
BaseLossP<-0

if(Chr_counts>60){Ploidy<-2}else{Ploidy<-1}

for(k in 1:dim(tmp)[[1]]){
  if((tmp[k,c_CN]==2*Ploidy)&&(tmp[k,c_BA]==1*Ploidy)){
	BaseP<-BaseP+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
  }
  if((tmp[k,c_CN]==1*Ploidy)&&(tmp[k,c_BA]==1*Ploidy)){
	BaseLossP<-BaseLossP+(SNP_annot[tmp[k,c_if],C_Pos]-SNP_annot[tmp[k,c_is],C_Pos])/1000
  }
}
BaseP<-round(BaseP/tt*100,0)
BaseLossP<-round(BaseLossP/tt*100,0)

tt<-which(tmp[,c_len]<20);if(length(tt)>0){tmp1<-tmp[-tt,]}
tmp1<-shrinkReprTMP(tmp1)

screen(4)

plot(c(1,56),c(1,65),type="n",main="Summary and Centromeres",cex.main=0.9,xlab="",ylab="",col.axis="white",xaxt="n",yaxt="n")

text(3,65,paste("Break counts >50  = ",as.character(dim(tmp1)[[1]]-41)),cex=0.8,pos=4)
text(3,62,paste("Chromosome counts = ",as.character(Chr_counts)),cex=0.8,pos=4)
text(3,59,paste("Centromere counts = ",as.character(Centr_counts)),cex=0.8,pos=4)
text(3,56,paste("DNA index         = ",as.character(DNAi)),cex=0.8,pos=4)

text(30,65,paste("Ploidy detected   = ",as.character(Ploidy*2)),cex=0.8,pos=4)
text(30,62,paste("% base state      = ",as.character(BaseP)),cex=0.8,pos=4)
text(30,59,paste("% '1' copy loss   = ",as.character(BaseLossP)),cex=0.8,pos=4)
text(30,56,paste("% accuired homo   = ",as.character(HomoP)),cex=0.8,pos=4)


rect(0,0,57,50,col="white",border="black")


for(p in 1:2){
lines(c(7+(p-1)*30,7+(p-1)*30),c(1,48),col="black")
lines(c(14+(p-1)*30,14+(p-1)*30),c(1,48),col="black")
lines(c(21+(p-1)*30,21+(p-1)*30),c(1,48),col="black")
lines(c(28+(1-1)*30,28+(1-1)*30),c(1,48),col="black")


text(3+(p-1)*30,48,"Chr",cex=0.8)
text(10+(p-1)*30,48,"Copy",cex=0.8)
text(17+(p-1)*30,48," B allele",cex=0.8)
text(24+(p-1)*30,48,"SNPs",cex=0.8)
}

Centro_breaks<-0

for(k in 1:dim(centr)[[1]]){

#k<-23
#k<-1
if(k<23){p<-1}else{p<-2}

if(trunc(centr[k,c_chr])==centr[k,c_chr]){
text(3+(p-1)*30,46*p-2*k-(p-1)*2,paste(as.character(trunc(centr[k,c_chr])),"p"),cex=0.7)
if((k<dim(centr)[[1]])&&(trunc(centr[k+1,c_chr])==centr[k,c_chr])&&(centr[k,c_CN]==centr[k+1,c_CN])&&
    ((centr[k,c_BA]==centr[k+1,c_BA])||(centr[k,c_BA]*centr[k+1,c_BA]==0))){
Centro_breaks<-Centro_breaks+1
}
}else{
text(3+(p-1)*30,46*p-2*k-(p-1)*2,paste(as.character(trunc(centr[k,c_chr])),"q"),cex=0.7)

}

text(10+(p-1)*30,46*p-2*k-(p-1)*2,as.character(centr[k,c_CN]),cex=0.7)
text(17+(p-1)*30,46*p-2*k-(p-1)*2,as.character(centr[k,c_BA]),cex=0.7)
text(24+(p-1)*30,46*p-2*k-(p-1)*2,as.character(centr[k,c_len]),cex=0.7)

}

#Centro_breaks<-(18-Centro_breaks)
#text(3,53,paste("Centromere breaks = ",as.character(Centro_breaks)),cex=0.8,pos=4)

close.screen(all = TRUE)
dev.off()

summary_info<-c(p_BAF,q_LRR,Centr_CL[5,4],(dim(tmp1)[[1]]-41),DNAi,Ploidy*2,Chr_counts)

out<-summary_info

}## summary plot function





###########################################################
####
####   Function UniteSEGtmp_new Uniting segments with similar attribution
####
###########################################################

UniteSEG_TMP<-function(tmp){

FL<-T
while(FL==T){
FL<-F

for(k in 1:(dim(tmp)[[1]]-1)){

	if((tmp[k,c_chr]==tmp[k+1,c_chr])&&(trunc(tmp[k,c_CN])==tmp[k,c_CN])
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k,c_BA]==tmp[k+1,c_BA])){

		tmp[k+1,c_is]<-tmp[k,c_is]
		
		w<-c(tmp[k+1,c_len],tmp[k,c_len])
		tmp[k+1,c_lrr]<-weighted.mean(c(tmp[k+1,c_lrr],tmp[k,c_lrr]),w)
		tmp[k+1,c_nC]<-weighted.mean(c(tmp[k+1,c_nC],tmp[k,c_nC]),w)

		tmp[k+1,c_CN1:(c_CN1+8)]<-(-1)
		tmp[k+1,c_CN1]<-tmp[k+1,c_CN]
		tmp[k+1,c_BA1]<-tmp[k+1,c_BA]
		tmp[k+1,c_rnk1]<-1

		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k+1,c_conf]<-max(tmp[k+1,c_conf],tmp[k,c_conf])	
		tmp[k,c_ind]<-0
	}else{

	if((tmp[k,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_CN]==0)&&(tmp[k+1,c_CN]==0)){

		tmp[k+1,c_is]<-tmp[k,c_is]
		

		w<-c(tmp[k+1,c_len],tmp[k,c_len])
		tmp[k+1,c_lrr]<-weighted.mean(c(tmp[k+1,c_lrr],tmp[k,c_lrr]),w)
		
		tmp[k+1,c_nC]<-weighted.mean(c(tmp[k+1,c_nC],tmp[k,c_nC]),w)

		tmp[k+1,c_CN1:(c_CN1+8)]<-(-1)
		tmp[k+1,c_CN1]<-tmp[k+1,c_CN]
		tmp[k+1,c_BA1]<-tmp[k+1,c_BA]
		tmp[k+1,c_rnk1]<-1

		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k+1,c_conf]<-max(tmp[k+1,c_conf],tmp[k,c_conf])	
		tmp[k,c_ind]<-0
	}
}
}#k

tt<-which(tmp[,c_ind]==0);if(length(tt)!=0){tmp<-tmp[-tt,];FL<-T}

} #FL

tmp<-round(tmp,4)
tmp[,c_nC]<-round(tmp[,c_nC],1)

out<-tmp

}# function





###########################################################
####
####   Function UniteSEGtmp_new Uniting segments with similar attribution
####
###########################################################

fitGermHomozyg<-function(tmp,Length){

FL<-T
while(FL==T){
FL<-F

tmp<-UniteSEG_TMP(tmp)

for(k in 1:dim(tmp)[[1]]){
	if((tmp[k,c_BA]==0)&&(tmp[k,c_len]>=1000)){tmp[k,c_BA]<-tmp[k,c_CN]}
}#k


for(k in 2:(dim(tmp)[[1]]-1)){

	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_BA]==0)
		&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_BA]==tmp[k+1,c_BA])&&(tmp[k+1,c_BA]!=0)
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k,c_len]<=Length)){

		tmp[k,c_BA]<-tmp[k+1,c_BA]
		FL<-T
	}
}#k


for(k in 2:(dim(tmp)[[1]]-1)){

	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_BA]==tmp[k,c_CN])
		&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_BA]==tmp[k+1,c_BA])&&(tmp[k+1,c_BA]!=0)
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k,c_len]<=Length)){

		tmp[k,c_BA]<-tmp[k+1,c_BA]
		FL<-T
	}
}#k


for(k in 2:(dim(tmp)[[1]]-1)){
	if(tmp[k,c_BA]==0){
		if((tmp[k,c_chr]==tmp[k+1,c_chr])
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k+1,c_BA]!=0)&&(tmp[k,c_len]<=Length)
		&&((tmp[k-1,c_CN]!=tmp[k,c_CN])||(tmp[k-1,c_chr]!=tmp[k,c_chr]))){

		tmp[k,c_BA]<-tmp[k+1,c_BA]
		FL<-T
		}

		if((tmp[k,c_chr]==tmp[k-1,c_chr])
		&&(tmp[k,c_CN]==tmp[k-1,c_CN])&&(tmp[k-1,c_BA]!=0)&&(tmp[k,c_len]<=Length)
		&&((tmp[k+1,c_CN]!=tmp[k,c_CN])||(tmp[k+1,c_chr]!=tmp[k,c_chr]))){

		tmp[k,c_BA]<-tmp[k-1,c_BA]
		FL<-T
		}
}


}#k

} #FL

out<-tmp

}# function



###########################################################
####
####   Function checkSmallHomozygosities_FinalPolish 
####    smoothing out small segments reported as germline himozygosities 
####
###########################################################


checkSmallHomozygosities_FinalPolish<-function(tmp,Length){

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])

TMP<-tmp

TMP<-fitGermHomozyg(TMP,Length)

for(k in 2:(dim(TMP)[[1]]-1)){
	if((TMP[k-1,c_chr]==TMP[k+1,c_chr])&&(max(TMP[k-1,c_len],TMP[k+1,c_len])/TMP[k,c_len]>3)&&(TMP[k,c_CN1+3]!=(-1))
		&&(TMP[k-1,c_CN]==TMP[k+1,c_CN])&&(TMP[k-1,c_BA]==TMP[k+1,c_BA])&&(TMP[k+1,c_BA]!=0)
		&&(TMP[k,c_CN]==TMP[k+1,c_CN])&&(TMP[k,c_BA]!=TMP[k+1,c_BA])&&(TMP[k,c_len]<=Length)
		){

		if((TMP[k,c_CN1]==TMP[k+1,c_CN])&&((TMP[k,c_BA1]==TMP[k+1,c_BA])||(TMP[k,c_BA1]==0))){TMP[k,c_BA]<-TMP[k,c_BA1]}
		if((TMP[k,c_CN1+3]==TMP[k+1,c_CN])&&((TMP[k,c_BA1+3]==TMP[k+1,c_BA])||(TMP[k,c_BA1+3]==0))){TMP[k,c_BA]<-TMP[k,c_BA1+3]}
		if((TMP[k,c_CN1+6]==TMP[k+1,c_CN])&&((TMP[k,c_BA1+6]==TMP[k+1,c_BA])||(TMP[k,c_BA1+6]==0))){TMP[k,c_BA]<-TMP[k,c_BA1+6]}
		
		tmp[TMP[k,c_ind],c_BA]<-TMP[k,c_BA]
		}
}#k


TMP<-fitGermHomozyg(TMP,Length)


tmp[,c_conf]<-0
for(k in 1:dim(TMP)[[1]]){
	tt<-which(tmp[,c_is]>=TMP[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=TMP[k,c_if])
	if(length(kk)>0){
		tmp[tt[kk],c_CN]<-TMP[k,c_CN]
		tmp[tt[kk],c_BA]<-TMP[k,c_BA]
		tmp[tt[kk],c_conf]<-k
	}}
}#k


out<-tmp

}#function



###########################################################
####
####   Function checkHomozygosities_FinalPolish 
####    In the case of the nearly pure tumor sample it is hard to distinguish
####      germline and aquired homozygosities.
####    By default they are often attributed as germline, which is not true
####     So, function corrects this for almost pure tumors
####
###########################################################


checkHomozygosities_FinalPolish<-function(tmp,Length,p_BAF){

tt<-which(tmp[,c_BA]==0)
if(length(tt)>0){germHomo<-sum(tmp[tt,c_len])/sum(tmp[,c_len])}else{germHomo<-0}


for(k in 1:dim(tmp)[[1]]){
	if((tmp[k,c_BA]==0)&&(tmp[k,c_len]>=1000)){tmp[k,c_BA]<-tmp[k,c_CN]}
}#k


if((p_BAF<0.25)||(germHomo>0.1)){

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_BA]==0)&&(tmp[k,c_len]>=Length)){tmp[k,c_BA]<-tmp[k,c_CN]}
}#k

}#if BAF

out<-tmp

}#function

###########################################################
####
####   Function checkSmallDiscrepancies looks for <100 SNPs segments which
####    differe less than 1 in c_cn, but 1 in c_CN 
####     and corrects them accordingly
####
###########################################################

checkHomozygBreaks_finalPolish<-function(tmp){


FL<-T
while(FL==T){
FL<-F
k<-100
for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_CN]>0)&&(tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_CN]!=tmp[k,c_CN])){
	if((tmp[k,c_BA]==0)||(tmp[k,c_CN]==tmp[k,c_BA])){
	if((abs(tmp[k,c_nC]-tmp[k+1,c_nC])<1)||(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<1)){
	if(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k-1,c_CN])){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T
	}}
	}}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
  if((tmp[k,c_CN]>0)&&(tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k-1,c_CN]!=tmp[k+1,c_CN])){
	if((tmp[k,c_BA]==0)||(tmp[k,c_CN]==tmp[k,c_BA])){
	if((abs(tmp[k,c_nC]-tmp[k+1,c_nC])<0.5)&&(abs(tmp[k+1,c_nC]-tmp[k+1,c_CN])<0.5)){
	if(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k+1,c_CN])){
		tmp[k,c_CN]<-tmp[k+1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T

	}}
	if((abs(tmp[k,c_nC]-tmp[k-1,c_nC])<0.5)&&(abs(tmp[k-1,c_nC]-tmp[k-1,c_CN])<0.5)){
	if(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k-1,c_CN])){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T

	}}
  }}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k+1,c_chr]!=tmp[k,c_chr])&&(tmp[k,c_CN]>0)&&((tmp[k,c_BA]==0)||(tmp[k,c_CN]==tmp[k,c_BA]))){
	if((tmp[k-1,c_chr]==tmp[k,c_chr])&&(tmp[k,c_CN]!=tmp[k-1,c_CN])&&(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<1)&&(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k-1,c_CN]))){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T
	}}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k-1,c_chr]!=tmp[k,c_chr])&&(tmp[k,c_CN]>0)&&((tmp[k,c_BA]==0)||(tmp[k,c_CN]==tmp[k,c_BA]))){
	if((tmp[k+1,c_chr]==tmp[k,c_chr])&&(tmp[k,c_CN]!=tmp[k+1,c_CN])&&(abs(tmp[k,c_nC]-tmp[k+1,c_nC])<1)&&(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k-1,c_CN]))){
		tmp[k,c_CN]<-tmp[k+1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T
	}}
}#k

}#FL

out<-tmp

}#function




###########################################################
####
####   Function checkSmallDiscrepancies looks for <100 SNPs segments which
####    differe less than 1 in c_cn, but 1 in c_CN 
####     and corrects them accordingly
####
###########################################################

checkSmallDiscrepancies_finalPolish<-function(tmp,Length){

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_len]<Length)&&(tmp[k,c_CN]>0)&&(max(tmp[k+1,c_len],tmp[k-1,c_len])>=2*Length)){
	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_CN]!=tmp[k,c_CN])){
	if((abs(tmp[k,c_nC]-tmp[k+1,c_nC])<1)||(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<1)){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		tmp[k,c_BA]<-tmp[k-1,c_BA]

	}}
	}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
  if((tmp[k,c_len]<Length)&&(tmp[k,c_CN]>0)&&(tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k-1,c_CN]!=tmp[k+1,c_CN])){
	if((tmp[k+1,c_len]>=2*Length)&&(abs(tmp[k,c_nC]-tmp[k+1,c_nC])<=1)&&(abs(tmp[k,c_nC]-tmp[k-1,c_nC])>1)){
		tmp[k,c_CN]<-tmp[k+1,c_CN]
		tmp[k,c_BA]<-tmp[k+1,c_BA]

	}
	if((tmp[k-1,c_len]>=2*Length)&&(abs(tmp[k,c_nC]-tmp[k+1,c_nC])>1)&&(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<=1)){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		tmp[k,c_BA]<-tmp[k-1,c_BA]

	}

  }
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k+1,c_chr]!=tmp[k,c_chr])&&(tmp[k,c_len]<Length)&&(tmp[k,c_CN]>0)){
	if((tmp[k-1,c_chr]==tmp[k,c_chr])&&(tmp[k-1,c_len]>=2*Length)&&(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<1)){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		tmp[k,c_BA]<-tmp[k-1,c_BA]

	}}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k-1,c_chr]!=tmp[k,c_chr])&&(tmp[k,c_len]<Length)&&(tmp[k,c_CN]>0)){
	if((tmp[k+1,c_chr]==tmp[k,c_chr])&&(tmp[k+1,c_len]>=2*Length)&&(abs(tmp[k,c_nC]-tmp[k+1,c_nC])<1)){
		tmp[k,c_CN]<-tmp[k+1,c_CN]
		tmp[k,c_BA]<-tmp[k+1,c_BA]

	}}
}#k


out<-tmp

}#function


